
%
% DISCLAIMER : 
% ============
% This software was developed at Physikalisch-Technische Bundesanstalt 
% (PTB). The software is made available "as is" free of cost. PTB assumes 
% no responsibility whatsoever for its use by other parties, and makes no 
% guarantees, expressed or implied, about its quality, reliability, safety,
% suitability or any other characteristic. In no event will PTB be liable 
% for any direct, indirect or consequential damage arising in connection 
% with the use of this software.
%
% Diese Software wurde in der Physikalisch-Technischen Bundesanstalt entwickelt. 
% Die Software wird unentgeldlich, wie sie ist, zur Verfuegung gestellt. Die PTB 
% uebernimmt keine Haftung, egal aus welchem Rechtsgrund, fuer die Nutzung der Software 
% durch Dritte und garantiert weder ausdruecklich noch konkludent fuer die Qualitaet, 
% Sicherheit, Zuverlaessigkeit, Tauglichkeit oder andere Merkmale der Software. 
% In keinem Fall uebernimmt die PTB die Haftung fuer direkte, indirekte Schaeden oder 
% fuer Folgeschaeden, die durch die Nutzung der Software entstehen. 
%
% This class defines a bandpass object.
%
% copyright F. Schmaehling & S. Eichstaedt (PTB) 2014

classdef BP < handle
   %bandpass object
   
   properties
      bandwidth = [];   % width of the bandpass
      delta = [];       % sampdedist of the bandpass
      b = [];           % values for the bandpass/slitfunction
      u_b = [];         % uncertainty for the bandpass/slitfunction
      options = [];     % some options for a bandpass object
   end
   
   methods
      function this = BP(bandwidth,delta,varargin)
         %constructor - create a bandpass object
         %possible constructors calls are
         %BP(bandwidth,delta)
         %BP(bandwith,delta,b)
         %BP(bandwith,delta,b,u_b)
         this.bandwidth = bandwidth;
         this.delta = delta;
         if nargin >= 3
            this.b = varargin{1};
            this.b = this.b(:)';
         end
         if nargin >= 4
            this.u_b = varargin{2};
            this.u_b = this.u_b(:)';
         end
      end
      
      function generate_b(this)
         %generate bandpass signal + uncertainty
         
         % definition of x-axis
         lambda = 0:this.delta:2*this.bandwidth;
         ind_kl = find(lambda < this.options.skew*this.bandwidth); % indices for increasing slope
         ind_gr = find(lambda >= this.options.skew*this.bandwidth);% indices for decreasing slope

         % definition of bandpass function    
         hat = zeros(size(lambda));
         hat(ind_kl) = lambda(ind_kl)/(this.options.skew*this.bandwidth^2);
         hat(ind_gr) = -lambda(ind_gr)/( (2-this.options.skew)*this.bandwidth^2 ) + 2/( (2-this.options.skew)*this.bandwidth );

         % definition of noise/measurement uncertainty
         noiseslit = zeros(size(lambda));
         noiseslit(ind_kl) = -((this.options.noise-this.options.noise_low)/(this.options.skew*this.bandwidth))*lambda(ind_kl) + this.options.noise;
         noiseslit(ind_gr) = ((this.options.noise-this.options.noise_low)/((2-this.options.skew)*this.bandwidth))*lambda(ind_gr) + (this.options.noise - 2*(this.options.noise-this.options.noise_low)/(2-this.options.skew));
         u_hat = hat.*noiseslit;
         epsilon = randn(size(lambda)).*u_hat;
         hat = epsilon.*u_hat+hat;

         this.b = hat;
      end
      
      function [b_uncert] = generate_b_uncertain(this)
         %definition of noise/measurement uncertainty bandpass (depent on upper and lower noise level)
         lambda = 0:this.delta:2*this.bandwidth;
         if (~isempty(this.u_b))
            epsilon = randn(size(lambda)).*this.u_b;
            b_uncert = epsilon.*this.u_b+this.b;
         else
            if (this.options.noise==0 || this.options.noise_low==0 || isempty(this.options.noise) || isempty(this.options.noise_low))
               b_uncert = this.b; %no uncertainty
            else
               ind_kl = find(lambda < this.options.skew*this.bandwidth); % indices for increasing slope
               ind_gr = find(lambda >= this.options.skew*this.bandwidth);% indices for decreasing slope
               noiseslit = zeros(size(lambda));
               noiseslit(ind_kl) = -((this.options.noise-this.options.noise_low)/(this.options.skew*this.bandwidth))*lambda(ind_kl) + this.options.noise;
               noiseslit(ind_gr) = ((this.options.noise-this.options.noise_low)/((2-this.options.skew)*this.bandwidth))*lambda(ind_gr) + (this.options.noise - 2*(this.options.noise-this.options.noise_low)/(2-this.options.skew));
               u_hat = this.b.*noiseslit;
               epsilon = randn(size(lambda)).*u_hat;
               b_uncert = epsilon.*u_hat+this.b;
            end
         end
      end
      
      function check_normalized(this)
         %check bandpass -> bandpass should be normalized (i.e. area under bandpass should be equal to 1)
         lambda = -this.bandwidth:this.delta:this.bandwidth;
         integral_value = trapz(lambda,this.b);
         if integral_value~=1
            %normalize
            this.b = this.b./integral_value;
         end
      end
      
      %setter/getter to control the inputs from the outside (and also from the inside)
      function set.bandwidth(this,bandwidth)
         assert(bandwidth>0,'error: bandwidth should > 0');
         this.bandwidth = bandwidth;
      end
      function bandwidth = get.bandwidth(this)
         bandwidth = this.bandwidth;
      end
      function set.delta(this,delta)
         assert(delta>0,'error: sampledist/stepsize bandpass should > 0');
         this.delta = delta;
      end
      function delta = get.delta(this)
         delta = this.delta;
      end
      function set.b(this,b)
         this.b = b(:)';
         this.check_normalized();
      end
      function b = get.b(this)
         b = this.b;
      end
      function set.u_b(this,u_b)
         this.u_b = u_b(:)';
      end
      function u_b = get.u_b(this)
         u_b = this.u_b;
      end
   end
   
end

