
%
% DISCLAIMER : 
% ============
% This software was developed at Physikalisch-Technische Bundesanstalt 
% (PTB). The software is made available "as is" free of cost. PTB assumes 
% no responsibility whatsoever for its use by other parties, and makes no 
% guarantees, expressed or implied, about its quality, reliability, safety,
% suitability or any other characteristic. In no event will PTB be liable 
% for any direct, indirect or consequential damage arising in connection 
% with the use of this software.
%
% Diese Software wurde in der Physikalisch-Technischen Bundesanstalt entwickelt. 
% Die Software wird unentgeldlich, wie sie ist, zur Verfuegung gestellt. Die PTB 
% uebernimmt keine Haftung, egal aus welchem Rechtsgrund, fuer die Nutzung der Software 
% durch Dritte und garantiert weder ausdruecklich noch konkludent fuer die Qualitaet, 
% Sicherheit, Zuverlaessigkeit, Tauglichkeit oder andere Merkmale der Software. 
% In keinem Fall uebernimmt die PTB die Haftung fuer direkte, indirekte Schaeden oder 
% fuer Folgeschaeden, die durch die Nutzung der Software entstehen. 
%
% This class define some options for the deconvolution approach using the
% Richardson-Lucy algorithm.
%
% As desrcibed in the header of the class RichLucyOOP.m, it is possible to
% define your own stopping criterium and regularization rule (both things can 
% be stored in the properties fh_StoppingRule and fh_Regularization as a function handle, 
% i.e. your must write your own stopping crit / regularization rule as a Matlab function and 
% put a handle of this function to the properties, for example see RichLucy.m or example_1.m)
%
% copyright F. Schmaehling & S. Eichstaedt (PTB) 2014

classdef RichLucyOptions < handle
   %RichLucy options - organized as a class
   
   properties
      max_iter = 5e3;   %maximal number of RL iterations
      min_iter = 5;     %minimal number of RL iterations  
      threshold = [];   %threshold for the constrain region where to judge RL
      verbose = 2;      %0, then no print or plot; 1, then print iteration result; 2, then also plot results of 1st step
      MC_counts = 1e4;  %default number of Monte-Carlo runs
      noise_up = [];    %upper noise level of the measured transmission curve
      noise_low = [];   %lower noise level of the measured transmission curve
   end
   
   properties (SetAccess = private)
      fh_StoppingRule = [];   %function handle, described the stopping rule for the RichLucy-Iteration
      fh_Regularization = []; %function handle, described a possible regularization scheme to smooth the RichLucy-Iterations
   end
   
   methods
      function this = RichLucyOptions()
          %constructor - constructor has no input parameters
         this.fh_StoppingRule = @this.curvature_crit;
      end
      
      function [im,RMS,curv] =  curvature_crit(this,course,M,min_iter,max_iter,BP,verbose)
         %calculate the point of maximum curvature
         %5 input parameters required (interface)
         RMS = zeros(size(course,1)-1,1);
         for k = 1:size(course,1)-1
            RMS(k,1) = sqrt(mean((course(k+1,:)-course(k,:)).^2));
         end
         q = linspace(0,log10(max_iter),max_iter);
         indqmin = find(q<=log10(min_iter),1,'last');
         qrms = spline(log10(1:max_iter),log10(RMS),q);
         first_derivative = gradient(qrms,q);
         snd_derivative = gradient(first_derivative,q);
         curv = abs(snd_derivative)./(1+first_derivative.^2).^(1.5);   
         qm= q(curv==max(curv(indqmin:end)));
         qmin = q(indqmin);
         if isempty(qm)
            if verbose
               fprintf(2,'Calculation of optimal stopping value failed.\nTaking tau=1e-4 as fallback.\n');
            end
            tau = 1e-3;
         else      
            im= find(log10(1:max_iter) <= qm, 1,'last');    
            tau = RMS(im);  
         end
         if isempty(tau)
            if verbose 
               fprintf(2,'Calculation of optimal stopping value failed.\nTaking tau=1e-4 as fallback.\n');
            end
            tau = 1e-3;
         end
         if verbose == 2
            %plot of the curvature-criterium
            figure(numel(get(0,'Children'))+1);clf;
            iters = 1:numel(RMS);        
            semilogx(iters,log10(RMS),10.^q,abs(curv));
            legend('log(rms) of change','curvature');
            line([10^qm, 10^qm],[min(min(log10(RMS)),min(curv)), max(max(log10(RMS)),max(curv))+1],'color','red');
            line([10^qmin, 10^qmin],[min(min(log10(RMS)),min(curv)), max(max(log10(RMS)),max(curv))+1],'color','k');
            xlabel('iter nr.');
            ylabel('log-rms value and its curvature');
         end
         if verbose
            fprintf('computed optimal number of iterations: %d\n',im-1);
         end
      end
      
      function Set_StoppingRule(this,fh_StoppingRule)
         %set a possible stopping rule for the Richardson-Lucy Algorithm 
         %-> a function handle
         disp('tip: StoppingRule must have one outputs [im]; im - the proposed stopping');
         this.fh_StoppingRule = fh_StoppingRule;
      end
      
      function Set_Regularization(this,fh_Regularization)
         %set a possible regularization rule for smoothing every step inside the Richardson-Lucy Algorithm 
         %-> a function handle
         this.fh_Regularization = fh_Regularization;
      end
      
      %setter/getter
      function set.max_iter(this,max_iter)
         assert(logical(max_iter>0 && max_iter>=this.min_iter),'error: choose a max_iter bigger than 0 and bigger than min_iter');
         this.max_iter = max_iter;
      end
      function max_iter = get.max_iter(this)
         max_iter = this.max_iter;
      end
      function set.min_iter(this,min_iter)
         assert((logical(min_iter>0)),'error: choose a min_iter bigger than 0');
         this.min_iter = min_iter;
      end
      function min_iter = get.min_iter(this)
         min_iter = this.min_iter;
      end
      function set.threshold(this,threshold)
         assert((logical(threshold>=0)),'error: threshold should be > 0');
         this.threshold = threshold;
      end
      function threshold = get.threshold(this)
         threshold = this.threshold;
      end
      function set.verbose(this,verbose)
         assert(logical(verbose>=0 && verbose <=2),'error: choose verbose between 0 and 2');
         this.verbose = verbose;
      end
      function verbose = get.verbose(this)
         verbose = this.verbose;
      end
      function fh_StoppingRule = get.fh_StoppingRule(this)
         %maybe some test or assertion
         fh_StoppingRule = this.fh_StoppingRule;
      end
      function fh_Regularization = get.fh_Regularization(this)
         %maybe some test or assertion
         fh_Regularization = this.fh_Regularization;
      end
      function set.noise_up(this,noise_up)
         assert(logical(noise_up>=0),'error: error: choose a upper noise level for the measured transmission curve >=0 (relative)');
         this.noise_up = noise_up;
      end
      function noise_up = get.noise_up(this)
         noise_up = this.noise_up;
      end
      function set.noise_low(this,noise_low)
         assert(logical(noise_low>=0),'error: error: choose a lower noise level for the measured transmission curve >=0 (relative)');
         this.noise_low = noise_low;
      end
      function noise_low = get.noise_low(this)
         noise_low = this.noise_low;
      end
   end
   
end

