
%
% DISCLAIMER : 
% ============
% This software was developed at Physikalisch-Technische Bundesanstalt 
% (PTB). The software is made available "as is" free of cost. PTB assumes 
% no responsibility whatsoever for its use by other parties, and makes no 
% guarantees, expressed or implied, about its quality, reliability, safety,
% suitability or any other characteristic. In no event will PTB be liable 
% for any direct, indirect or consequential damage arising in connection 
% with the use of this software.
%
% Diese Software wurde in der Physikalisch-Technischen Bundesanstalt entwickelt. 
% Die Software wird unentgeldlich, wie sie ist, zur Verfuegung gestellt. Die PTB 
% uebernimmt keine Haftung, egal aus welchem Rechtsgrund, fuer die Nutzung der Software 
% durch Dritte und garantiert weder ausdruecklich noch konkludent fuer die Qualitaet, 
% Sicherheit, Zuverlaessigkeit, Tauglichkeit oder andere Merkmale der Software. 
% In keinem Fall uebernimmt die PTB die Haftung fuer direkte, indirekte Schaeden oder 
% fuer Folgeschaeden, die durch die Nutzung der Software entstehen. 
%
% For deconvolution the Richardson-Lucy approach is used as described in
%  (1) Richardson W.H., Bayesian-Based Iterative Method of Image Restoration, 
%      Journal of the Optical Society of America, vol. 62 nr. 1, 1972
%  (2) Lucy L.B., An iterative technique for the rectification of observed distributions, 
%      The Astronomical Journal, vol. 79, nr. 6, 1974
%  (3) Jansson, P.A., Deconvolution of Images and Spectra, Second Edition, Academic Press 1996
%
% Here, per default, the RL algorithm is applied in two steps adaptively. In the first
% step a large number of iterations is carried out. From this result an
% optimal number is determined using the L-curve method. In a second step
% the RL algorithm is carried out again using the results from the 1st
% step. For more details see
%
%      Eichstaedt S., Schmaehling F., Wuebbeler G., Anhalt K., Buenger L., Krueger U. & Elster C.,
%      Comparison of the Richardson-Lucy method and a classical approach for spectrometer bandwidth correction,
%      Metrologia**, vol. 50, 2013
%
% Moreover, for the advanced user it is possible to defined a own user 
% defined stopping rule for the iterative Richardson-Lucy deconvolution 
% approach and a user defined regularization rule that can be used to smooth 
% every iteration of the iterative approch (in more detail see class RichLuyOptions.m)
%
% For example on usage of this class take a look at RichLucy.m or example_1.m
%
% copyright F. Schmaehling & S. Eichstaedt (PTB) 2014

classdef RichLucy_OOP < handle
   
   properties (SetAccess = private)
      lambda = []       %wavelength steps
      M = [];           %measured transmission curve
      u_M = [];         %uncertainty for the measured transmission curve
      delta = [];       %sampledist of the measurement points
      my_BP = [];       %bandpass object
      RL_Options = [];  %possible Options for the RichLucy-Algorithm
      R = [];           %reference transmission curve (optional)
   end
   
   methods
      function this = RichLucy_OOP(lambda_M,M,u_M,bandwidth,b_delta,varargin)
         %constructor
         %possible constructors calls are
         %RichLucy_OOP(lambda_M,M,u_M,bandwidth,b_delta)
         %RichLucy_OOP(lambda_M,M,u_M,bandwidth,b_delta,varargin)
         %varargin{1} ... slitfunction (bandpass)
         %varargin{2} ... uncertainty slitfunction (bandpass)
         %varargin{3} ... bandpass options (BPOptions)
         %varargin{4} ... Richardson-Lucy options (RichLucyOptions)
         this.my_BP = BP(bandwidth,b_delta);             %object of class BP
         this.lambda = lambda_M;
         this.M = M;
         this.u_M = u_M;
         this.RL_Options = RichLucyOptions();            %object of class RichLucyOptions
         if nargin >= 6
            if ~isempty(varargin{1})
               this.my_BP.b = varargin{1};
            else
               if ~isempty(varargin{3})
                  this.my_BP.options = varargin{3};
                  this.my_BP.generate_b;
               else
                  this.my_BP.options = BPOptions;        %default options for the bandpass
                  this.my_BP.generate_b;                 %generate (synthetic) bandpass
               end
            end
         else
            this.my_BP.options = BPOptions;              %default options for the bandpass
            this.my_BP.generate_b;                       %generate (synthetic) bandpass
         end
         if nargin >= 7
            if ~isempty(varargin{2})
               this.my_BP.u_b = varargin{2};
            end
         end
         if nargin >= 8
            if ~isempty(varargin{4})
               this.my_BP.options = varargin{3};
            end
         end
         if nargin >= 9
            if ~isempty(varargin{4})
               this.RL_Options = varargin{4};
            end
         end
      end
      
      %methods
      function Shat = Start_RL(this,varargin)
         %Start a deconvolution process based on the Richardson-Lucy algorithm 
         my_M = this.M;
         my_b = this.my_BP.b;
         if nargin >= 2
            my_M = varargin{1};
            if nargin >= 3
               my_b = varargin{2};
            end
         end
         % impulse invariance technique for discretization
         bd = my_b*this.my_BP.delta;
         % flip slitfunction to have convolution model    
         bdtilde = fliplr(bd);
         % ---
         % 1st step - do maximal RL iterations
         % ---
         if (isempty(this.RL_Options.threshold))
            ind = 1:numel(my_M);
         else
            ind = find(my_M>this.RL_Options.threshold);
         end
         if (this.RL_Options.verbose && (nargin == 1))
            fprintf(1,'RL iteration ');
         end
         clear course; clear course_full;
         [course,course_full] = this.RichLucyIteration(my_M,bdtilde,my_M,ind,(this.RL_Options.verbose>=1 && nargin==1));
         if (this.RL_Options.verbose>=1 && (nargin==1))
           fprintf(' Done.\n');
         end
         if ~isempty(ind)
            M_tmp = my_M(ind);
         else
            M_tmp = my_M;
         end
         if (nargin==1)
            verbose_local = this.RL_Options.verbose;
         else
            verbose_local = 0;
         end
         % ---
         % 2nd step - determine optimal number of iterations
         % ---
         if this.RL_Options.verbose && nargin == 1
            if isempty(this.RL_Options.fh_StoppingRule)
               fprintf(1,'RL method calculating optimal stop criterion (using curvature).\n');
            else
               fprintf(1,'RL method calculating optimal stop criterion (using user defined stopping criterium).\n');
            end
            if ~isempty(this.RL_Options.fh_Regularization)
               fprintf(1,'A user defined regularization rule was used.\n');
            end
         end
         [im] = this.RL_Options.fh_StoppingRule(course,M_tmp,this.RL_Options.min_iter,this.RL_Options.max_iter,this.my_BP,verbose_local); 
         if (this.RL_Options.verbose>=1 && (nargin==1))
           fprintf('Done.\n');
         end
         r = im;
         if (im+1>size(course_full,1))
            Shat = course_full(end,:);
         else
            Shat = course_full(im+1,:);
         end
         if this.RL_Options.verbose>=2 && (nargin==1)
            fprintf('RL iteration finished');
            if r < this.RL_Options.max_iter            
               fprintf(' after %d iterations.\n',r-1);
            else
               fprintf(', because max number of iterations reached (%d).\n',this.RL_Options.max_iter);
            end
         end
         if (this.RL_Options.verbose == 2 && (nargin==1))
            figure(numel(get(0,'Children'))+1);clf;
            plot(this.lambda,this.M,'-b');
            hold on;
            plot(this.lambda,Shat,'-g');
            if (~isempty(this.R))
               plot(this.lambda,this.R,'--k');
            end
         end
         clear course; clear course_full;
      end
      
      function [course,course_full] = RichLucyIteration(this,Shat,bdtilde,my_M,ind,verboseflag)
         %this is the core of the Richarsdon-Lucy approach for deconvolution
         %for more information see: 
         %W.H. Richardson, 'Bayesian-based iterative method of image restoration', J.Opt.Soc.Am. 62, 1972
         %L.B. Lucy, 'An iterative technique for the rectification of observed distributions', Astron.J. 79, 1974
         %L.B. Lucy, 'Astronomical inverse problems', Rev.Mod.Astron. 7, 1994
         tmp_RL_Options = this.RL_Options; this.RL_Options = tmp_RL_Options; %to avoid memory problems, when max_iter is to large -> see getter property
         anz = this.RL_Options.max_iter + 1;
         if isempty(ind)
            course = zeros(anz,numel(Shat));
         else
            course = zeros(anz,numel(Shat(ind)));
         end
         course_full = zeros(anz,numel(Shat));
         for k = 1:anz
            tmp1 = conv(Shat,bdtilde,'same');
            tmp1 = my_M./tmp1;
            tmp1(isnan(tmp1) | isinf(tmp1)) = 0;             % safeguard
            tmp2 = conv(tmp1,fliplr(bdtilde),'same');
            Shat_new = Shat.*tmp2;
            Shat_new(isnan(Shat_new) | isinf(Shat_new)) = 0; % safeguard
            if ~isempty(this.RL_Options.fh_Regularization)
               Shat_new = Shat_new.*this.RL_Options.fh_Regularization(Shat_new,Shat);
            end
            if isempty(ind)
               course(k,:) = Shat_new;
            else
               course(k,:) = Shat_new(ind);
            end
            course_full(k,:) = Shat_new;
            Shat = Shat_new;
            if (mod(k,round(anz*0.05))==0 && verboseflag)
               fprintf(1,'%s','.');
            end
         end
      end
      
      function [mean_Shad,u_Shad,MC_Shad] = Start_RichLucyUncertainty(this)
         %Start a deconvolution process with uncertainty calculation via Monte-Carlo
         assert(~(isempty(this.my_BP.options) && isempty(this.my_BP.u_b)) && ~(isempty(this.u_M) && (isempty(this.RL_Options.noise_up) || isempty(this.RL_Options.noise_low))),'to do uncertainty evaluation, at least a uncertainty for the measured transmission curve and/or the slitfunction (bandpass) are/is needed');
         MC_Shad = zeros(this.RL_Options.MC_counts,numel(this.M));
         if this.RL_Options.verbose>=2 
            fprintf('Starting Monte-Carlo.\n');
         end
         outver = ver;
         bflag = 0;
         for k = 1:numel(outver)
            if strcmp(outver(k).Name,'Parallel Computing Toolbox')
               bflag = 1;
            end
         end
         if bflag
            if (matlabpool('size')==0)
               disp('To make Monte-Carlo evaluation faster, it is possible to start the matlabpool, y/n?');
               s = input('Continue?   ','s');
               bf1 = 0;
               if (~isempty(s))
                  if (strcmp(s,'y'))
                     bf1 = 1;
                     matlabpool open;
                     b_mc = zeros(this.RL_Options.MC_counts,numel(this.my_BP.b));
                     M_mc = zeros(this.RL_Options.MC_counts,numel(this.M));
                     if (isempty(this.u_M))
                        if (~isempty(this.RL_Options.noise_up) && ~isempty(this.RL_Options.noise_low))
                           u_M_tmp = this.generate_M_uncertain();
                        else
                           u_M_tmp = zeros(size(this.M));
                        end
                     else
                        u_M_tmp = this.u_M;
                     end
                     disp('Initializing');
                     for k = 1:this.RL_Options.MC_counts
                        b_mc(k,:) = this.my_BP.generate_b_uncertain();
                        M_mc(k,:) = randn(size(this.M)).*u_M_tmp+this.M; %implied this.u_M is iid
                     end
                     disp('Running');
                     pctRunOnAll javaaddpath java;
                     if (this.RL_Options.MC_counts < 100)
                        progressStepSize = 1;
                     else
                        progressStepSize = 10;
                     end
                     ppm = ParforProgMon('please wait, Monte-Carlo evaluation is running ...',this.RL_Options.MC_counts,progressStepSize,400,80);
                     parfor k = 1:this.RL_Options.MC_counts
                        %draw new slitfunction (bandpass)
                        %draw new measured transmission curve
                        MC_Shad(k,:) = Start_RL(this,M_mc(k,:),b_mc(k,:));
                        if mod(k,progressStepSize)==0
                           ppm.increment();
                        end
                     end
                     ppm.delete();
                     disp('Finish');
                  end
               end
               if (bf1 == 0)
                  hw = waitbar(0,'please wait, Monte-Carlo evaluation is running ...');
                  for k = 1:this.RL_Options.MC_counts
                     %draw new slitfunction (bandpass)
                     %draw new measured transmission curve
                     b_mc = this.my_BP.generate_b_uncertain();
                     if (isempty(this.u_M))
                        if (~isempty(this.RL_Options.noise_up) && ~isempty(this.RL_Options.noise_low))
                           M_mc = randn(size(this.M)).*this.generate_M_uncertain()+this.M;
                        else
                           M_mc = randn(size(this.M)).*zeros(size(this.M))+this.M; %implied this.u_M is iid
                        end
                     else
                        M_mc = randn(size(this.M)).*this.u_M+this.M; %implied this.u_M is iid
                     end
                     MC_Shad(k,:) = Start_RL(this,M_mc,b_mc);
                     if (mod(k,(this.RL_Options.MC_counts-1)*0.05)==0 && this.RL_Options.verbose>=2)
                        fprintf(1,'%s','.');
                     end
                     waitbar(k/this.RL_Options.MC_counts);
                  end
                  close(hw);
               end
            else
               b_mc = zeros(this.RL_Options.MC_counts,numel(this.my_BP.b));
               M_mc = zeros(this.RL_Options.MC_counts,numel(this.M));
               if (isempty(this.u_M))
                  if (~isempty(this.RL_Options.noise_up) && ~isempty(this.RL_Options.noise_low))
                     u_M_tmp = this.generate_M_uncertain();
                  else
                     u_M_tmp = zeros(size(this.M));
                  end
               else
                  u_M_tmp = this.u_M;
               end 
               disp('Initializing');
               for k = 1:this.RL_Options.MC_counts
                  b_mc(k,:) = this.my_BP.generate_b_uncertain();
                  M_mc(k,:) = randn(size(this.M)).*u_M_tmp+this.M; %implied this.u_M is iid
               end
               disp('Running');
               pctRunOnAll javaaddpath java;
               if (this.RL_Options.MC_counts < 100)
                  progressStepSize = 1;
               else
                  progressStepSize = 10;
               end
               ppm = ParforProgMon('please wait, Monte-Carlo evaluation is running ...',this.RL_Options.MC_counts,progressStepSize,400,80);
               parfor k = 1:this.RL_Options.MC_counts
                  %draw new slitfunction (bandpass)
                  %draw new measured transmission curve
                  MC_Shad(k,:) = Start_RL(this,M_mc(k,:),b_mc(k,:));
                  if mod(k,progressStepSize)==0
                     ppm.increment();
                  end
               end
               ppm.delete();
               disp('Finish');
            end
         else
            hw = waitbar(0,'please wait, Monte-Carlo evaluation is running ...');
            for k = 1:this.RL_Options.MC_counts
               %draw new slitfunction (bandpass)
               %draw new measured transmission curve
               b_mc = this.my_BP.generate_b_uncertain();
               if (isempty(this.u_M))
                  if (~isempty(this.RL_Options.noise_up) && ~isempty(this.RL_Options.noise_low))
                     M_mc = randn(size(this.M)).*this.generate_M_uncertain()+this.M;
                  else
                     M_mc = randn(size(this.M)).*zeros(size(this.M))+this.M; %implied this.u_M is iid
                  end
               else
                  M_mc = randn(size(this.M)).*this.u_M+this.M; %implied this.u_M is iid
               end
               MC_Shad(k,:) = Start_RL(this,M_mc,b_mc);
               if (mod(k,(this.RL_Options.MC_counts-1)*0.05)==0 && this.RL_Options.verbose>=2)
                  fprintf(1,'%s','.');
               end
               waitbar(k/this.RL_Options.MC_counts);
            end
            close(hw);
         end
         if this.RL_Options.verbose>=2 
            fprintf('Done.\n');
         end
         mean_Shad = mean(MC_Shad);
         u_Shad = std(MC_Shad);
         if (this.RL_Options.verbose == 2)
            figure(numel(get(0,'Children'))+1);clf;
            plot(this.lambda,this.M,'-b');
            hold on;
            plot(this.lambda,mean_Shad,'-g');
            plot(this.lambda,mean_Shad+u_Shad,'--r');
            plot(this.lambda,mean_Shad-u_Shad,'--r');
            if (~isempty(this.R))
               plot(this.lambda,this.R,'--k');
            end
         end
      end
      
      function bV = check_equi(this,lambda)
         %check for equidistant lambda (wavelength steps)
         bV = logical(numel(unique(round(diff(lambda).*1e10)/1e10)==1));
         if bV
            this.delta = unique(round(diff(lambda).*1e10)/1e10);
         end
      end
      
      function bV = check_strictly_monotonic(this,lambda)
         %check for strictly monot. increasing wavelength steps
         bV = logical(all(round(diff(lambda)*1e6)/1e6 > 0));
      end
      
      function check_StepLength(this)
         %check step length (equal sample distances in bandpass and measured signal)
         if (isempty(this.delta))
            this.delta = this.my_BP.delta;
            %interpolate (measured transmission curve)
            new_lambda = this.lambda(1):this.my_BP.delta:this.lambda(end);
            this.M = pchip(this.lambda,this.M,new_lambda);
            this.lambda = new_lambda;
         end
         if (this.delta ~= this.my_BP.delta)
            this.delta = this.my_BP.delta;
            %interpolate (measured transmission curve)
            new_lambda = this.lambda(1):this.my_BP.delta:this.lambda(end);
            this.M = pchip(this.lambda,this.M,new_lambda);
            this.lambda = new_lambda;
         end
      end
      
      function M = EastMeas(this,R)
         %build a synthetic measured transmission curve
         %EastMeas(R) ... R signal that should be used to simulate a measurement
         assert(all(R>=0),'error: negative values in signal that should be used to simulate a measurement',0); 
         this.R = R;
         D1 = bsxfun(@plus,this.lambda,(-this.my_BP.bandwidth:this.my_BP.delta:this.my_BP.bandwidth)');
         D2 = pchip(this.lambda,R,D1);
         D3 = D2.*repmat(this.my_BP.b',1,size(D1,2));
         M = sum((D3(1:end-1,:)+D3(2:end,:))./2.*this.my_BP.delta); %measurement reference signal 
      end
      
      function uncM = generate_M_uncertain(this)
         %calculate uncertainty for the measured signal dependent on upper and lower noise level
         uncM = (-(this.RL_Options.noise_up-this.RL_Options.noise_low)*( this.M/max(this.M) ) + this.RL_Options.noise_up).*this.M;
      end
      
      %setter (to control the input to the properties)
      function set.lambda(this,lambda)
         assert(this.check_equi(lambda),'error: wavelength steps not equidistant',0);
         assert(this.check_strictly_monotonic(lambda),'error: wavelength steps not strictly monot. increasing',0); 
         this.lambda = lambda;
      end
      function set.M(this,M)
         assert(~isempty(M),'no values inside the measurement signal');
         assert(all(M>=0),'error: negative values in measured transmission curve',0); 
         this.M = M;
         this.check_StepLength();
      end
      function set.delta(this,delta)
         assert(delta>0,'error: sampledist of the measurement points should be > 0');
         this.delta = delta;
      end
      function set.RL_Options(this,RL_Options)
         if (~isempty(this.M))
            [usr,~] = memory;
            b_calc_possible = (2*RL_Options.max_iter<floor((usr.MaxPossibleArrayBytes/8)./numel(this.M)));
            if (~b_calc_possible)
               RL_Options.max_iter = floor((usr.MaxPossibleArrayBytes/8)./numel(this.M)/2)-1;
               fprintf('%s\n%s%d\n\n','Memory problems can be arise, because number of max. iterations are to high.',...
                  'The number of maximal iteration is set to the largest possible value ',RL_Options.max_iter);
            end
            this.RL_Options = RL_Options;
         else
            this.RL_Options = RL_Options;
         end
      end
   end
   
end

