%
% DISCLAIMER : 
% ============
% This software was developed at Physikalisch-Technische Bundesanstalt 
% (PTB). The software is made available "as is" free of cost. PTB assumes 
% no responsibility whatsoever for its use by other parties, and makes no 
% guarantees, expressed or implied, about its quality, reliability, safety,
% suitability or any other characteristic. In no event will PTB be liable 
% for any direct, indirect or consequential damage arising in connection 
% with the use of this software.
%
% Diese Software wurde in der Physikalisch-Technischen Bundesanstalt entwickelt. 
% Die Software wird unentgeldlich, wie sie ist, zur Verfuegung gestellt. Die PTB 
% uebernimmt keine Haftung, egal aus welchem Rechtsgrund, fuer die Nutzung der Software 
% durch Dritte und garantiert weder ausdruecklich noch konkludent fuer die Qualitaet, 
% Sicherheit, Zuverlaessigkeit, Tauglichkeit oder andere Merkmale der Software. 
% In keinem Fall uebernimmt die PTB die Haftung fuer direkte, indirekte Schaeden oder 
% fuer Folgeschaeden, die durch die Nutzung der Software entstehen. 
%
% Algorithm based on Richardson-Lucy deconvolution as described in
%  (1) Richardson W.H., Bayesian-Based Iterative Method of Image Restoration, 
%      Journal of the Optical Society of America, vol. 62 nr. 1, 1972
%  (2) Lucy L.B., An iterative technique for the rectification of observed distributions, 
%      The Astronomical Journal, vol. 79, nr. 6, 1974
%  (3) Jansson, P.A., Deconvolution of Images and Spectra, Second Edition, Academic Press 1996
%
% Here the Richardson-Lucy iteration is applied for several iterations. After each
% block of iterations, the estimation result is plotted and the user is asked to decide
% whether to stop the iterations or continue.
%
% Input parameters
% b           ... spectrometer slitfunction
% delta       ... sampledist of the measurement points
% M           ... measured transmission curve
% iter_step   ... number of iterations until plot of intermediate results
%
%
% copyright F. Schmaehling & S. Eichstaedt (PTB) 2014
%
% usage:
%
% Shat = RichLucy_interactive(b,delta,M,iter_step)

function Shat = RichLucy_interactive(b,delta,M,iter_step)

% make sure that the dimension is correct
   b = b(:)';

% impulse invariance technique for discretization
   bd = b*delta;
% flip slitfunction to have convolution model    
   bdtilde = fliplr(bd);

   if (nargin < 3)
      Shat = [];
      fprintf(2,'Not enough input parameters.\n');
      return;
   end
% set default values
   if nargin < 4
       iter_step = 250;
   end
   fignr = 1001;

   fprintf('\n');
   fprintf('-------------------------------------------------------\n');
   fprintf('Interactive Richardson-Lucy algorithm started\n');
   fprintf('\t initial number of iterations: %d\n\n',iter_step);
    
   % initial estimate is equal to measurement
   Shat = M;
   
   %---   
   clear my_RLO;
   %create an RichLucyOptions object
   my_RLO = RichLucyOptions;
   my_RLO.max_iter = iter_step;
   my_RLO.min_iter = 1;
   my_RLO.threshold = 1e-3;
   my_RLO.verbose = 0;

   clear my_RichLucy;
   %create an RichLucy_OOP object
   if (mod(numel(b),2)==0)
      bandwidth = (numel(b)-1)/2*delta;
   else
      bandwidth = (numel(b)-1)/2*delta;
   end
   my_RichLucy = RichLucy_OOP(100:delta:100+delta*(numel(M)-1),M,[],bandwidth,delta,b,[],[],my_RLO);
   %---
   
   course = [];
   cont = true;
   k = 1;
   im = [];
   while cont
      
      clear course_tmp;
      my_RLO.max_iter = iter_step;
      [~,course_tmp] = my_RichLucy.RichLucyIteration(Shat,bdtilde,M,M>=my_RLO.threshold,0);
      course = [course;course_tmp];
      Shat = course(end,:);
      
      M_tmp = M(M>=my_RLO.threshold);
      max_iter = size(course,1)-1;
      [im,RMS,curv] = my_RichLucy.RL_Options.fh_StoppingRule(course,M_tmp,1,max_iter,my_RichLucy.my_BP,0); 
      
      figure(fignr-1);clf;
      q = linspace(0,log10(max_iter),max_iter);
      iters = 1:numel(RMS);        
      semilogx(iters,log10(RMS),10.^q,abs(curv));
      legend('log(rms) of change','curvature');
      xlabel('iter nr.');
      ylabel('log-rms value and its curvature');
      
      figure(fignr);clf;
      lambda = 0:delta:(numel(M)-1)*delta;
      plot(lambda,M,lambda,Shat,lambda,conv(Shat,bdtilde,'same'));
      legend('measured spectrum','current est.','est. meas.');
      xlabel('wavelength'); ylabel('amplitude');
      ylim([0 1.1]);
      
      disp('Continue Richardson-Lucy iterations?');
      disp('Enter number of iterations or press enter to continue. Enter `no` to stop.');
      s = input('Continue?   ','s');
      if isempty(s)
         fprintf('\nRichardson-Lucy iterations are continued ...\n\n');
      elseif strcmp(s(1),'n')
         cont = false;
         fprintf('\nRichardson-Lucy iterations stopped upon user request.\n\n');
      else
         try 
            iter_step = str2double(s);
            if (isnan(iter_step))
               fprintf('\nAn error occured while trying to convert your input to double.\n');
               fprintf('Iterations are stopped and the last result is returned\n\n');
               cont = false;
            end
            fprintf('\nRichardson-Lucy iterations are continued ...\n\n');
         catch
            fprintf('\nAn error occured while trying to convert your input to double.\n');
            fprintf('Iterations are stopped and the last result is returned\n\n');
            cont = false;
         end 
      end
      
      k = k + 1;
      
   end
   
   if (~isempty(im))
      fprintf('optimal stopping after %d iterations.\n',im);
      indqmin = find(q<=log10(1),1,'last');
      qm = q(curv==max(curv(indqmin:end)));
      qmin = q(indqmin);
      figure(fignr-1);hold on;
      line([10^qm, 10^qm],[min(min(log10(RMS)),min(curv)), max(max(log10(RMS)),max(curv))+1],'color','red');
      line([10^qmin, 10^qmin],[min(min(log10(RMS)),min(curv)), max(max(log10(RMS)),max(curv))+1],'color','k');
   end
   
end
