%
% DISCLAIMER : 
% ============
% This software was developed at Physikalisch-Technische Bundesanstalt 
% (PTB). The software is made available "as is" free of cost. PTB assumes 
% no responsibility whatsoever for its use by other parties, and makes no 
% guarantees, expressed or implied, about its quality, reliability, safety,
% suitability or any other characteristic. In no event will PTB be liable 
% for any direct, indirect or consequential damage arising in connection 
% with the use of this software.
%
% Diese Software wurde in der Physikalisch-Technischen Bundesanstalt entwickelt. 
% Die Software wird unentgeldlich, wie sie ist, zur Verfuegung gestellt. Die PTB 
% uebernimmt keine Haftung, egal aus welchem Rechtsgrund, fuer die Nutzung der Software 
% durch Dritte und garantiert weder ausdruecklich noch konkludent fuer die Qualitaet, 
% Sicherheit, Zuverlaessigkeit, Tauglichkeit oder andere Merkmale der Software. 
% In keinem Fall uebernimmt die PTB die Haftung fuer direkte, indirekte Schaeden oder 
% fuer Folgeschaeden, die durch die Nutzung der Software entstehen. 
%
% Algorithm based on Richardson-Lucy deconvolution as described in
%  (1) Richardson W.H., Bayesian-Based Iterative Method of Image Restoration, 
%      Journal of the Optical Society of America, vol. 62 nr. 1, 1972
%  (2) Lucy L.B., An iterative technique for the rectification of observed distributions, 
%      The Astronomical Journal, vol. 79, nr. 6, 1974
%  (3) Jansson, P.A., Deconvolution of Images and Spectra, Second Edition, Academic Press 1996
%
% Here the RL algorithm is applied in two steps adaptively. In the first
% step a large number of iterations is carried out. From this result an
% optimal number is determined using the L-curve method. In a second step
% the RL algorithm is carried out again using the results from the 1st
% step. 
%
% Input parameters
%	b           ... spectrometer slitfunction
%	delta       ... sampledist of the measurement points
%	M           ... measured transmission curve
%  max_iter    ... maximal number of RL iterations
%
% copyright F. Schmaehling & S. Eichstaedt (PTB) 2012
%
% usage:
%
% [ Shat,rms(change),curv(rms),tau] = RL_2step(b,delta,M,max_iter)


function [Shat,RMS] = RichLucy_stepwise(b,delta,Shat,max_iter,varargin)

% make sure that the dimension is correct
    b = b(:)';

% impulse invariance technique for discretization
    bd = b*delta;
% flip slitfunction to have convolution model    
    bdtilde = fliplr(bd);

    RMS = zeros(max_iter,1);
    
    if nargin > 4
        if ~isempty(varargin{1})
            M = varargin{1};
        end
    else
        M = Shat;
    end
    
    ind = 1:numel(M);
  
    r = 1;
% additional iterations
    fprintf(1,'RL method is running: ');
    while r <= max_iter
        % RL step
        tmp1 = conv(Shat,bdtilde,'same');
        tmp1 = M./tmp1;
        tmp1(isnan(tmp1) | isinf(tmp1)) = 0; % safeguard
        tmp2 = conv(tmp1,fliplr(bdtilde),'same');
        Shat_new = Shat.*tmp2;
        Shat_new(isnan(Shat_new) | isinf(Shat_new)) = 0;

        RMS(r) = sqrt(mean( (Shat_new(ind)-Shat(ind)).^2 ));        
        Shat = Shat_new;
        r = r+1;
        if (mod(r,max_iter*0.05)==0)
           fprintf(1,'%s','.');
        end
    end
    fprintf(' Maximum number of iterations reached.\n'); 
    
end
