
%
% DISCLAIMER : 
% ============
% This software was developed at Physikalisch-Technische Bundesanstalt 
% (PTB). The software is made available "as is" free of cost. PTB assumes 
% no responsibility whatsoever for its use by other parties, and makes no 
% guarantees, expressed or implied, about its quality, reliability, safety,
% suitability or any other characteristic. In no event will PTB be liable 
% for any direct, indirect or consequential damage arising in connection 
% with the use of this software.
%
% Diese Software wurde in der Physikalisch-Technischen Bundesanstalt entwickelt. 
% Die Software wird unentgeldlich, wie sie ist, zur Verfuegung gestellt. Die PTB 
% uebernimmt keine Haftung, egal aus welchem Rechtsgrund, fuer die Nutzung der Software 
% durch Dritte und garantiert weder ausdruecklich noch konkludent fuer die Qualitaet, 
% Sicherheit, Zuverlaessigkeit, Tauglichkeit oder andere Merkmale der Software. 
% In keinem Fall uebernimmt die PTB die Haftung fuer direkte, indirekte Schaeden oder 
% fuer Folgeschaeden, die durch die Nutzung der Software entstehen. 
%
% The proposed stopping rule for the Richardson-Lucy approach is described in detail in the
% Metrologia journal
%    "Comparison of the Richardson-Lucy method and a classical approach for spectrometer
%    bandwidth correction", Eichstaedt, Schmaehling, Wuebbeler, Anhalt, Buenger, Krueger
%    and Elster; Metrologia 2012
%
% copyright F. Schmaehling & S. Eichstaedt (PTB) 2014

function [im,RMS,curv] =  StoppingRule_CurvatureCrit(course,M,min_iter,max_iter,BP,verbose)
   
   RMS = zeros(size(course,1)-1,1);
   for k = 1:size(course,1)-1
      RMS(k,1) = sqrt(mean((course(k+1,:)-course(k,:)).^2));
   end
   q = linspace(0,log10(max_iter),max_iter);
   indqmin = find(q<=log10(min_iter),1,'last');
   qrms = spline(log10(1:max_iter),log10(RMS),q);
   first_derivative = gradient(qrms,q);
   snd_derivative = gradient(first_derivative,q);
   curv = abs(snd_derivative)./(1+first_derivative.^2).^(1.5);   
   qm= q(curv==max(curv(indqmin:end)));
   qmin = q(indqmin);
   if isempty(qm)
      if verbose
         fprintf(2,'Calculation of optimal stopping value failed.\nTaking tau=1e-4 as fallback.\n');
      end
      tau = 1e-3;
   else      
      im= find(log10(1:max_iter) <= qm, 1,'last');    
      tau = RMS(im);  
   end
   if isempty(tau)
      if verbose 
         fprintf(2,'Calculation of optimal stopping value failed.\nTaking tau=1e-4 as fallback.\n');
      end
      tau = 1e-3;
   end
   if verbose == 2
      %plot of the curvature-criterium
      figure(numel(get(0,'Children'))+1);clf;
      iters = 1:numel(RMS);        
      semilogx(iters,log10(RMS),10.^q,abs(curv));
      legend('log(rms) of change','curvature');
      line([10^qm, 10^qm],[min(min(log10(RMS)),min(curv)), max(max(log10(RMS)),max(curv))+1],'color','red');
      line([10^qmin, 10^qmin],[min(min(log10(RMS)),min(curv)), max(max(log10(RMS)),max(curv))+1],'color','k');
      xlabel('iter nr.');
      ylabel('log-rms value and its curvature');
   end
   if verbose
      fprintf('computed optimal number of iterations: %d\n',im-1);
   end
   
end

