% DISCLAIMER : 
% ============
% This software was developed at Physikalisch-Technische Bundesanstalt 
% (PTB). The software is made available "as is" free of cost. PTB assumes 
% no responsibility whatsoever for its use by other parties, and makes no 
% guarantees, expressed or implied, about its quality, reliability, safety,
% suitability or any other characteristic. In no event will PTB be liable 
% for any direct, indirect or consequential damage arising in connection 
% with the use of this software.
%
%
% This is an example to demonstrate application of the provided software. 
% A measurement of a Gaussian-shaped spectrum with a triangular bandpass 
% function is simulated. The DO3 and DO5 methods are applied for bandwidth 
% correction for various wavelength step size values to demonstrate
% the problem of noise amplification. Results are compared to that of the 
% Richardson-Lucy method.
%
%
% copyright F. Schmaehling & S. Eichstaedt (PTB) 2012
%
%

%% Definition of paramaters

rng(10);

noise_up = 2e-2;		% max relative noise level of measured signal
noise_low = 3e-3;   % min relative noise level of measured signal

noiseb_up = 2e-2;      % max relative noise level of bandpass measurement
noiseb_low = 1e-3;  % min relative noise level of bandpass measurement

bandwidth = 20;  	% half width of the triangular monochromator slitfunction
delta_b = .1;       % sampledist for monochromator (in nm)
skew  = 1;	        % parameter for asymmetry of bandpass (symmetry for skew=1.00)
delta_M = delta_b;        % sampledist for measurement points (in nm)
mu    = 500;        % mean of Gaussian pulse (simulated input spectrum)
FWHM = 25;  		% FWHM value of Gaussian pulse (simulated input spectrum)
sigma  = sqrt(FWHM^2/(6*log(2)));

lambda_min = 400;   % min wavelength in spectrum measurement
lambda_max = 600;   % max wavelength in spectrum measurement
lambda_Mess = lambda_min+bandwidth:delta_M:lambda_max-bandwidth;

Sfun   = @(lam) exp(-(lam-mu).^2/(2*sigma^2));
S = Sfun(lambda_Mess);


fprintf('\n##### Bandwidth correction demo ##### \nRichardson-Lucy, DO3 and DO5 in comparison\n');
fprintf('\nSimulating measurement with parameters:\n');
fprintf('bandpass function FWHM: %g\nbandpass function skewness factor: %g\n',bandwidth,skew);
fprintf('bandpass function step size: %g\n',delta_b);
fprintf('measurement wavelength step size: %g\n',delta_M);
fprintf('input spectrum peak value: 1\ninput spectrum FWHM: %g\n',FWHM);
fprintf('interval of measurement: [%g nm, %g nm]\n',lambda_min,lambda_max);
fprintf('\n');

%% simulation of measurement

  % generation of bandpass function
    [b,lambda_b] = simulate_bandpass(bandwidth,delta_b,skew,noiseb_up,noiseb_low);
  % calculation of wavelength scale correction 
    lambda_centroid = trapz(lambda_b-bandwidth,(lambda_b-bandwidth).*b);
    
% simulation of the measurement          
     M = simulate_measurement(Sfun,lambda_Mess,b,bandwidth,delta_b,noise_low,noise_up);
     
     
    figure (1);clf;
    [b,lambda_b] = simulate_bandpass(bandwidth,delta_b,skew,0,0);
    plot(lambda_b,b);
    xlabel('wavelength in nm','fontsize',16);
    ylabel('bandpass function amplitude in a.u.','fontsize',16);
    set(gca,'fontsize',14);
  
%% bandwidth correction - DO methods
fprintf('bandwidth correction using DO methods (for various wavelength step size values)\n');

deltas = [delta_M,1:15];

for k=1:numel(deltas)
    
    delta = deltas(k);

    inds = 1:delta*10:numel(M);
    
    ShatDO5 = DO5(lambda_b-lambda_centroid,b,bandwidth,delta,M(inds));
    ShatDO3 = DO3(lambda_b-lambda_centroid,b,bandwidth,delta,M(inds));
         
    DO5rms(k) = sqrt(mean( (ShatDO5-S(inds)).^2 ));
    DO3rms(k) = sqrt(mean( (ShatDO3-S(inds)).^2 ));
        
    figure (2);clf;
    set(gcf,'position',[100 200 1400 600]);
    subplot(121);
    plot(lambda_Mess,[S;M],lambda_Mess(inds),ShatDO3,'g');
    xlabel('wavelength in nm','fontsize',16);
    ylabel('spectrum in a.u.','fontsize',16);
    xlim([430 570]);
    legend('input','measured',sprintf('DO3 est. (delta_M=%g)',delta));
    set(gca,'fontsize',12);
    subplot(122);
    plot(lambda_Mess,[S;M],lambda_Mess(inds),ShatDO5,'r');
    xlabel('wavelength in nm','fontsize',16);
    ylabel('spectrum in a.u.','fontsize',16);
    xlim([430 570]);
    legend('input','measured',sprintf('DO5 est.(delta_M=%g)',delta));
    set(gca,'fontsize',12);
    pause(1);
end
 
%% comparison to Richardson-Lucy
fprintf('bandwidth correction using Richardson-Lucy method\n');

for k=1:numel(deltas)    
    delta = deltas(k);
    inds = 1:delta*10:numel(M);
    MRL = interp1(lambda_Mess(inds),M(inds),lambda_Mess,'pchip'); % pretend that only the M(inds) values are known
    ShatRL = RichLucy(b,delta_M,MRL);
    RLrms(k) = sqrt(mean( (ShatRL-S).^2 ));    
end
 
%% plotting the comparison  

    figure(3);clf;
    semilogy(deltas,[DO5rms;DO3rms;RLrms],'d-');
    legend('DO5 rms error','DO3 rms error','RichLucy rms error');
    xlabel('wavelength step size');
    ylabel('rms error');
    
    
