% DISCLAIMER : 
% ============
% This software was developed at Physikalisch-Technische Bundesanstalt 
% (PTB). The software is made available "as is" free of cost. PTB assumes 
% no responsibility whatsoever for its use by other parties, and makes no 
% guarantees, expressed or implied, about its quality, reliability, safety,
% suitability or any other characteristic. In no event will PTB be liable 
% for any direct, indirect or consequential damage arising in connection 
% with the use of this software.
%
%
% This is an example to demonstrate application of the provided software. 
% A measurement of a Gaussian-shaped spectrum with a triangular bandpass 
% function is simulated. The Richardson-Lucy method is applied for bandwidth correction.
%
% To show a simple example of the usage of the Richardson-Lucy algorithm
% and the related software (function RichLucy is a wrapper -> inside the a object of 
% type RichLucy_OOP is instantiated, method for deconvolution using the Richardson-Lucy 
% approach and uncertainty calculations via Monte-Carlo are encapsulated in the object)
%
% copyright F. Schmaehling & S. Eichstaedt (PTB) 2014

%% Definition of paramaters
rng(10);

noise_up = 2e-2;		% max relative noise level of measured signal
noise_low = 3e-3;    % min relative noise level of measured signal

noiseb_up = 2e-2;    % max relative noise level of bandpass measurement
noiseb_low = 1e-3;   % min relative noise level of bandpass measurement

bandwidth = 40;      % half width of the triangular monochromator slitfunction
delta_b = .1;        % sampledist for monochromator (in nm)
skew  = 1;           % parameter for asymmetry of bandpass (symmetry for skew=1.00)
delta_M = delta_b;   % sampledist for measurement points (in nm)
mu    = 500;         % mean of Gaussian pulse (simulated input spectrum)
FWHM = 10;           % FWHM value of Gaussian pulse (simulated input spectrum)
sigma  = sqrt(FWHM^2/(6*log(2)));

lambda_min = 400;    % min wavelength in spectrum measurement
lambda_max = 600;    % max wavelength in spectrum measurement
lambda_Mess = lambda_min+bandwidth:delta_M:lambda_max-bandwidth;

Sfun   = @(lam) exp(-(lam-mu).^2/(2*sigma^2));
S = Sfun(lambda_Mess);

fprintf('\n##### Bandwidth correction demo ##### \napplication of Richardson-Lucy method\n');
fprintf('\nSimulating measurement with parameters:\n');
fprintf('bandpass function FWHM: %g\nbandpass function skewness factor: %g\n',bandwidth,skew);
fprintf('bandpass function step size: %g\n',delta_b);
fprintf('measurement wavelength step size: %g\n',delta_M);
fprintf('input spectrum peak value: 1\ninput spectrum FWHM: %g\n',FWHM);
fprintf('interval of measurement: [%g nm, %g nm]\n',lambda_min,lambda_max);

%% simulation of measurement

% generation of bandpass function
b = simulate_bandpass(bandwidth,delta_b,skew,noiseb_up,noiseb_low);

% simulation of the measurement          
M = simulate_measurement(Sfun,lambda_Mess,b,bandwidth,delta_b,noise_low,noise_up);
    
%% application of Richardson-Lucy method (automatic)
% A difficulty with large number of iterations of the Richardson-Lucy method is that noise is amplified strongly.
% This results in rapid changes in the evolution of the estimation progress and hence, causes strong peaks in the
% corresponding curvature values for higher iteration numbers. Thus, the maximum number of iterations should not be too large.

% General outline:
% 1. start with large value of max_iter (e.g. 1000)
% 2. have a look at evaluation of estimates (output_flag=2)
% 3. decrease max_iter, if necessary, such that only reasonable results are covered

max_iter = 5e3;     % maximum number of iterations
min_iter = 1;       % minimum number of iterations
threshold = 1e-4;   % wavelengths at which to judge reconstruction progress
output_flag = 2;    % verbose

ShatRL = RichLucy(b,delta_b,M,max_iter,min_iter,threshold,output_flag);

%%

figure (1);clf;
[b,lambda_b] = simulate_bandpass(bandwidth,delta_b,skew,0,0);
plot(lambda_b,b);
xlabel('wavelength in nm','fontsize',16);
ylabel('bandpass function amplitude in a.u.','fontsize',16);
set(gca,'fontsize',14);


figure (2);clf;
plot(lambda_Mess,[S;M;ShatRL]);
xlabel('wavelength in nm','fontsize',16);
ylabel('spectrum in a.u.','fontsize',16);
xlim([430 570]);
legend('input spectrum','measured spectrum','Rich-Lucy estimate');
set(gca,'fontsize',14);
