% DISCLAIMER : 
% ============
% This software was developed at Physikalisch-Technische Bundesanstalt 
% (PTB). The software is made available "as is" free of cost. PTB assumes 
% no responsibility whatsoever for its use by other parties, and makes no 
% guarantees, expressed or implied, about its quality, reliability, safety,
% suitability or any other characteristic. In no event will PTB be liable 
% for any direct, indirect or consequential damage arising in connection 
% with the use of this software.
%
%
% This is an example to demonstrate application of the provided software. 
% A measurement of a Gaussian-shaped spectrum with a triangular bandpass 
% function is simulated. The Richardson-Lucy method is applied for bandwidth correction.
%
% To illustrate the Richardson-Lucy algorithm stepwise and interactive
%
% copyright F. Schmaehling & S. Eichstaedt (PTB) 2014

%% Definition of paramaters
rng(10);

noise_up = 2e-2;		% max relative noise level of measured signal
noise_low = 3e-3;    % min relative noise level of measured signal

noiseb_up = 2e-2;    % max relative noise level of bandpass measurement
noiseb_low = 1e-3;   % min relative noise level of bandpass measurement

bandwidth = 40;      % half width of the triangular monochromator slitfunction
delta_b = .1;        % sampledist for monochromator (in nm)
skew  = 1;           % parameter for asymmetry of bandpass (symmetry for skew=1.00)
delta_M = delta_b;   % sampledist for measurement points (in nm)
mu    = 500;         % mean of Gaussian pulse (simulated input spectrum)
FWHM = 10;           % FWHM value of Gaussian pulse (simulated input spectrum)
sigma  = sqrt(FWHM^2/(6*log(2)));

lambda_min = 400;    % min wavelength in spectrum measurement
lambda_max = 600;    % max wavelength in spectrum measurement
lambda_Mess = lambda_min+bandwidth:delta_M:lambda_max-bandwidth;

Sfun   = @(lam) exp(-(lam-mu).^2/(2*sigma^2));
S = Sfun(lambda_Mess);

fprintf('\n##### Bandwidth correction demo ##### \napplication of Richardson-Lucy method\n');
fprintf('\nSimulating measurement with parameters:\n');
fprintf('bandpass function FWHM: %g\nbandpass function skewness factor: %g\n',bandwidth,skew);
fprintf('bandpass function step size: %g\n',delta_b);
fprintf('measurement wavelength step size: %g\n',delta_M);
fprintf('input spectrum peak value: 1\ninput spectrum FWHM: %g\n',FWHM);
fprintf('interval of measurement: [%g nm, %g nm]\n',lambda_min,lambda_max);

%% simulation of measurement

% generation of bandpass function
b = simulate_bandpass(bandwidth,delta_b,skew,noiseb_up,noiseb_low);

% simulation of the measurement          
M = simulate_measurement(Sfun,lambda_Mess,b,bandwidth,delta_b,noise_low,noise_up);

%% application of Richardson-Lucy method (interactive)

Shat = RichLucy_interactive(b,delta_b,M);

