% DISCLAIMER : 
% ============
% This software was developed at Physikalisch-Technische Bundesanstalt 
% (PTB). The software is made available "as is" free of cost. PTB assumes 
% no responsibility whatsoever for its use by other parties, and makes no 
% guarantees, expressed or implied, about its quality, reliability, safety,
% suitability or any other characteristic. In no event will PTB be liable 
% for any direct, indirect or consequential damage arising in connection 
% with the use of this software.
%
% This is an example to demonstrate application of the provided software. 
% A measurement of a Gaussian-shaped spectrum with a triangular bandpass 
% function is simulated. All methods, DO3, DO5 and Richardson-Lucy are applied
% for bandwidth correction. Uncertainties are evaluated using Monte Carlo.
%
% When your are able to use the Matlab Parallel Toolbox, open the
% matlabpool via matlabpool open X with X workers before you start this
% script (max. number on workers currently 12 (Matlab R2012b)) 
%
% copyright F. Schmaehling & S. Eichstaedt (PTB) 2014

%% Definition of paramaters for simulated measurement
rng(10);

noise_up = 2e-2;     % max relative noise level of measured signal
noise_low = 3e-3;    % min relative noise level of measured signal

noiseb_up = 2e-2;    % max relative noise level of bandpass measurement
noiseb_low = 1e-3;   % min relative noise level of bandpass measurement

bandwidth = 20;      % half width of the triangular monochromator slitfunction
delta_b = .1;        % sampledist for monochromator (in nm)
skew  = 1;           % parameter for asymmetry of bandpass (symmetry for skew=1.00)
delta_M = 5;         % sampledist for measurement points (in nm)
mu    = 500;         % mean of Gaussian pulse (simulated input spectrum)
FWHM = 25;           % FWHM value of Gaussian pulse (simulated input spectrum)
sigma  = sqrt(FWHM^2/(6*log(2)));

lambda_min = 400;    % min wavelength in spectrum measurement
lambda_max = 600;    % max wavelength in spectrum measurement
lambda_Mess = lambda_min+bandwidth:delta_M:lambda_max-bandwidth;

Sfun   = @(lam) exp(-(lam-mu).^2/(2*sigma^2));
S = Sfun(lambda_Mess);


fprintf('\n##### Bandwidth correction demo ##### \nRichardson-Lucy, DO3 and DO5 in comparison\n');
fprintf('\nSimulating measurement with parameters:\n');
fprintf('bandpass function FWHM: %g\nbandpass function skewness factor: %g\n',bandwidth,skew);
fprintf('bandpass function step size: %g\n',delta_b);
fprintf('measurement wavelength step size: %g\n',delta_M);
fprintf('input spectrum peak value: 1\ninput spectrum FWHM: %g\n',FWHM);
fprintf('interval of measurement: [%g nm, %g nm]\n',lambda_min,lambda_max);
fprintf('\n');

%% bandwidth correction and evaluation of uncertainties

runs = 1e3;       % number of runs for Monte Carlo

% set up matrices to store Monte Carlo results
SHM   = zeros(runs,numel(lambda_Mess));
SHDO3 = zeros(runs,numel(lambda_Mess));
SHDO5 = zeros(runs,numel(lambda_Mess));
SHRL  = zeros(runs,numel(lambda_Mess));

% Monte Carlo simulations

bprint = true;
fprintf('Pleae wait - Monte-Carlo simulation for the DOx-methods is running\n');
if (matlabpool('size') > 0)
   fprintf('using Matlab Parallel Toolbox with %d workers\n',matlabpool('size'));
   bprint = false;
end
parfor k=1:runs

% generation of bandpass function
    [b,lambda_b] = simulate_bandpass(bandwidth,delta_b,skew,noiseb_up,noiseb_low);
% calculation of wavelength scale correction 
    lambda_centroid = trapz(lambda_b-bandwidth,(lambda_b-bandwidth).*b);
    
% simulation of the measurement          
    SHM(k,:) = simulate_measurement(Sfun,lambda_Mess,b,bandwidth,delta_b,noise_low,noise_up);
  
% estimation/reconstruction of input spectrum

% differential operator approach
    SHDO5(k,:) = DO5(lambda_b-lambda_centroid,b,bandwidth,delta_M,SHM(k,:));        
    SHDO3(k,:) = DO3(lambda_b-lambda_centroid,b,bandwidth,delta_M,SHM(k,:));                
         
end
fprintf('Monto-Carlo simulation for the DOx-methods finished\n');

%%
%---
% Richardson-Lucy
[b,~] = simulate_bandpass(bandwidth,delta_b,skew,noiseb_up,noiseb_low);
[~,u_M,M_mean] = simulate_measurement(Sfun,lambda_Mess,b,bandwidth,delta_b,noise_low,noise_up);

clear my_RLO;
%create an RichLucyOptions object
my_RLO = RichLucyOptions;
my_RLO.max_iter = 1e2;
my_RLO.min_iter = 2;
my_RLO.threshold = 1e-3;
my_RLO.verbose = 1;
my_RLO.MC_counts = runs;
my_RLO.noise_up = noise_up;
my_RLO.noise_low = noise_low;

clear my_BPO;
%create an BPOPtions object
my_BPO = BPOptions;
my_BPO.skew = 1;
my_BPO.noise = noiseb_up;
my_BPO.noise_low = noiseb_low;

if (mod(numel(b),2)==0)
   bandwidth = (numel(b)-1)/2*delta_b;
else
   bandwidth = (numel(b)-1)/2*delta_b;
end

clear my_RichLucy;
%Create a RicLucy_OOP object
my_RichLucy = RichLucy_OOP(lambda_Mess,M_mean,[],bandwidth,delta_b,b,[],my_BPO,my_RLO);
fprintf('Monte-Carlo simulation for the Richardson-Lucy.\n');
%start uncertainty evaluation via member method Start_RichLucyUncertainty of class RichLucy_OOP.m
[~,~,Shat_mc_tmp] = my_RichLucy.Start_RichLucyUncertainty;
lambda_M_RL = lambda_Mess(1):delta_b:lambda_Mess(end);
Shat_mc = zeros(runs,numel(lambda_Mess));
for k = 1:runs
   Shat_mc(k,:) = pchip(lambda_M_RL,Shat_mc_tmp(k,:),lambda_Mess);
end
%---

%%
% calculation of mean and standard uncertainty
ShatM  = M_mean;
uShatM  = u_M;

ShatRL = mean(Shat_mc);
uShatRL = std(Shat_mc);

ShatDO3= mean(SHDO3);
uShatDO3= std(SHDO3);
    
ShatDO5= mean(SHDO5);
uShatDO5= std(SHDO5);


%%
%plot some things

figure (1);clf;
[b,lambda_b] = simulate_bandpass(bandwidth,delta_b,skew,0,0);
plot(lambda_b,b);
xlabel('wavelength in nm','fontsize',16);
ylabel('bandpass function amplitude in a.u.','fontsize',16);
set(gca,'fontsize',14);


figure (20);
plot(lambda_Mess,S,'-k',lambda_Mess,ShatM,'--k',...
lambda_Mess,ShatRL,'-b',...
lambda_Mess,ShatDO3,'-g',lambda_Mess,ShatDO5,'-r');
xlabel('wavelength in nm','fontsize',16);
ylabel('spectrum in a.u.','fontsize',16);
xlim([430 570]);
legend('input spectrum','measured spectrum','Rich-Lucy','DO3','DO5');
set(gca,'fontsize',14);


figure(30);clf;
hold on;
errorbar(lambda_Mess,abs(S-ShatM),uShatM,'-k','linewidth',2);
errorbar(lambda_Mess,abs(S-ShatRL),uShatRL,'-b','linewidth',2);
errorbar(lambda_Mess,abs(S-ShatDO3),uShatDO3,'-g','linewidth',2);
errorbar(lambda_Mess,abs(S-ShatDO5),uShatDO5,'-r','linewidth',2);
axis([430 570 -0.15 0.2]);
xlabel('wavelength in nm','fontsize',16);
ylabel('amplitude difference in a.u.','fontsize',16);
set(gca,'fontsize',18);
title('uncertainty of difference $\hat{S}-S$','interpreter','latex');
legend('no correction','Rich-Lucy','DO3','DO5');    
hold off;

    
