% DISCLAIMER : 
% ============
% This software was developed at Physikalisch-Technische Bundesanstalt 
% (PTB). The software is made available "as is" free of cost. PTB assumes 
% no responsibility whatsoever for its use by other parties, and makes no 
% guarantees, expressed or implied, about its quality, reliability, safety,
% suitability or any other characteristic. In no event will PTB be liable 
% for any direct, indirect or consequential damage arising in connection 
% with the use of this software.
% 
% 
% This is an example to demonstrate application of the provided software. 
% A measurement of a reference spectrum with a measured bandpass 
% function is simulated. All methods, DO3, DO5 and Richardson-Lucy are applied
% for bandwidth correction.
%
%
% copyright F. Schmaehling and S. Eichstaedt (PTB) 2014
% 

%% load measurement data
   clear;

   bfile = importdata('measured_bandpass.txt');
   Rfile = importdata('reference_spectrum.txt');

   lambda_Mess = Rfile.data(:,1);
   lambda_b    = bfile.data(:,1);

   Ref = Rfile.data(:,2);
   b   = bfile.data(:,2);

   delta_M = lambda_Mess(2)-lambda_Mess(1);
   delta_b = lambda_b(2)-lambda_b(1);
    
    
%% simulation of measurement

   M = conv(Ref,flipud(b)*delta_b,'same');

   figure (1);clf;
   subplot(121);
   plot(lambda_Mess,[Ref,M]);
   legend('reference spectrum','simulated measurement','location','south');
   subplot(122);
   plot(lambda_b,b);
   legend('bandpass function');
    
   
%%	bandwidth correction (DO methods)
   fprintf('\nbandwidth correction using DO methods\n');

   scale_DO3 = 3; % wavelength step size for DO3 = scale_DO3*delta_M
   scale_DO5 = 5; % wavelength step size for DO5 = scale_DO5*delta_M

   inds_DO3 = 1:scale_DO3:numel(lambda_Mess);
   inds_DO5 = 1:scale_DO5:numel(lambda_Mess);

   ShatDO3 = DO3(lambda_b,b,0,scale_DO3*delta_M,M(inds_DO3));  
   ShatDO5 = DO5(lambda_b,b,0,scale_DO5*delta_M,M(inds_DO5));  

%% bandwidth correction (Richardson-Lucy method)    
   fprintf('\nbandwidth correction using Richardson-Lucy method\n');

   %---   
   clear my_RLO;
   %create an RichLucyOptions object
   my_RLO = RichLucyOptions;
   my_RLO.max_iter = 5000;
   my_RLO.min_iter = 5;
   my_RLO.verbose = 2;
   
   %example for the integration of a user defined stoppning criterium and/or regularization rule
   %StoppingRule_EEm_Crit.m is a user defined stopping criterium saved as a
   %matlab function -> a handle to this function is given to the RichLucyOptions object "my_RLO"
   %your own routine should be saved as a matlab function with input and 
   %output parameters similar to StoppingRule_EEM_Crit.m or StoppingRule_CurvatureCrit.m
%    my_RLO.Set_StoppingRule(@StoppingRule_CurvatureCrit);
   my_RLO.Set_StoppingRule(@StoppingRule_EEM_Crit);
   %a users defined regularization rule is given (as a function handle) to the RichLucyOptions object "my_RLO"
   %your own routine should be saved as a matlab function with input and 
   %output parameters similar to Regularization_Tikhonov.m
   %to have freedom in formulating your own regularization rule
   %signal_new (after a new Rich-Lucy step) and signal_old (Rich-Lucy step before the new) 
   %should be your minimal input interface
   tau_reg = 5e-4;
   my_RLO.Set_Regularization(@(signal_new,signal_old)Regularization_Tikhonov(signal_new,signal_old,delta_b,tau_reg));
   
   clear my_RichLucy;
   %create an RichLucy_OOp object
   if (mod(numel(b),2)==0)
      bandwidth = (numel(b)-1)/2*delta_b;
   else
      bandwidth = (numel(b)-1)/2*delta_b;
   end
   my_RichLucy = RichLucy_OOP(lambda_Mess,M,[],bandwidth,delta_b,b,[],[],my_RLO);
   %to start RichLucy deconvolution -> call method Start_RL of the RichLucy_OOP object
   ShatRL = my_RichLucy.Start_RL;

%%
   figure (2);clf;
   plot(lambda_Mess,Ref,'-b',...
   lambda_Mess(inds_DO3),ShatDO3,'-g',lambda_Mess(inds_DO5),ShatDO5,'-r',...
   lambda_Mess,ShatRL,'-k');
   legend('reference','DO3 estimate','DO5 estimate','Rich-Lucy estimate');    
   xlabel('wavelength in nm','fontsize',12);
   ylabel('amplitude in a.u.','fontsize',12);
   set(gca,'fontsize',12);

   figure (3);clf;
   plot(lambda_Mess(inds_DO3),Ref(inds_DO3)-ShatDO3','-g',...
   lambda_Mess(inds_DO5),Ref(inds_DO5)-ShatDO5','-r',...
   lambda_Mess,Ref-ShatRL','-k');
   legend('error of DO3','error of DO5','error of Rich-Lucy');
   xlabel('wavelength in nm','fontsize',12);
   ylabel('estimation error in a.u.','fontsize',12);
   set(gca,'fontsize',12);

   figure (2);clf;
   plot(lambda_Mess,Ref,'-b',...
   lambda_Mess(inds_DO3),ShatDO3,'-g',lambda_Mess(inds_DO5),ShatDO5,'-r',...
   lambda_Mess,ShatRL,'-k');
   legend('reference','DO3 estimate','DO5 estimate','Rich-Lucy estimate');    
   xlabel('wavelength in nm','fontsize',12);
   ylabel('amplitude in a.u.','fontsize',12);
   set(gca,'fontsize',12);

    
