# -*- coding: utf-8 -*-
"""
Copyright S. Eichstaedt, F. Schmaehling (PTB) 2013
sascha.eichstaedt@ptb.de


This software is licensed under the BSD-like license:

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in 
     the documentation and/or other materials provided with the distribution.

This software was developed at Physikalisch-Technische Bundesanstalt.
The software is made available 'as is' free of cost. PTB assumes
no responsibility whatsoever for its use by other parties, and makes no
guarantees, expressed or implied, about its quality, reliability, safety,
suitability or any other characteristic. In no event will PTB be liable
for any direct, indirect or consequential damage arising in connection
with the use of this software.

The algorithm used here is based on Richardson-Lucy deconvolution as 
described in
Eichstaedt, Schmaehling, Wuebbeler, Anhalt, Buenger, Krueger & Elster
'Comparison of the Richardson-Lucy method and a classical approach for 
spectrometer bandwidth correction', Metrologia vol. 50, 2013

"""


version = 2.0

import sys

from PyQt4 import QtCore, QtGui

from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as Canvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
from matplotlib.figure import Figure

from os.path import isfile

import numpy as np
from scipy.interpolate import interp1d

import RichLucy as RL
import stopping_rules as sr
import smoothing_methods as sm

display_text_output = True
maxcpu_for_uncertainty_calc = 3

class PlotWindow(QtGui.QMainWindow):
    """
    Class which generates a separate window with a figure as a child of some
    main window. In this way, all plots are closed when the main window is closed.            
    """
    
    title = 'Separate plot'
    closed = QtCore.pyqtSignal()
    
    def __init__(self,parent=None):
        QtGui.QMainWindow.__init__(self,parent)
        self.parent = parent
        self.setWindowTitle(self.title)
        self.main = QtGui.QWidget()
        self.fig = Figure((7,6))
        self.canvas = Canvas(self.fig)
        self.canvas.setParent(self.main)
        self.axes = self.fig.add_subplot(111)
        self.nav = NavigationToolbar(self.canvas,self.main)
        self.layout = QtGui.QVBoxLayout()
        self.layout.addWidget(self.canvas)
        self.layout.addWidget(self.nav)
        self.main.setLayout(self.layout)
        self.setCentralWidget(self.main)
        
    def changeTitle(self,text):        
        self.setWindowTitle(text)
        
    def plot(self,x,y,x2=None,y2=None,col1='b',col2='r'):
        if isinstance(x2,np.ndarray):
            self.axes.plot(x,y,col1,x2,y2,col2)
            self.canvas.draw()
        elif isinstance(x,np.ndarray):
            self.axes.plot(x,y,col1)
            self.canvas.draw()
        else:
            self.close()
            
    def semilogx(self,x,y):
        self.axes.semilogx(x,y)
        self.canvas.draw() 
        
    def axvline(self,value):
        self.axes.axvline(value,color="k",linestyle="--")
        self.canvas.draw()
        
    def closeEvent(self,event):
        self.closed.emit()
        event.accept()
   
   
class BrowseRL(QtGui.QMainWindow):
    
    closed = QtCore.pyqtSignal()
    
    def __init__(self,lM,M,lS,SH,iter0=1,parent=None):
        QtGui.QMainWindow.__init__(self,parent)
        self.parent = parent
        self.main = QtGui.QWidget()
        self.fig = Figure((10,8))
        self.canvas = Canvas(self.fig)
        self.canvas.setParent(self.main)
        self.axes = self.fig.add_subplot(111)
        self.layout = QtGui.QVBoxLayout()
        self.layout.addWidget(self.canvas)
        self.setWindowTitle("Browsing the RichLucy estimates...")
       
        self.slider = QtGui.QSlider()
        self.slider.setOrientation(QtCore.Qt.Horizontal)
        self.slider.setFocusPolicy(QtCore.Qt.NoFocus)
        self.slider.setRange(0,SH.shape[0])
        self.slider.valueChanged.connect(self.update)
       
        self.layout.addWidget(self.slider)
        
        self.main.setLayout(self.layout)
        self.setCentralWidget(self.main)
        
        self.axes.plot(lM,M,'bo',alpha=0.6,label="measured spectrum")
        self.iter_txt = self.axes.text(lM[0]+5,max(M)*0.9,"iter %d" % iter0,fontsize=16)
        self.axes.set_xlabel("wavelength")
        self.axes.set_ylabel("spectrum amplitude")
        self.axes.set_ylim(0,M.max()*1.2)
        self.lS = lS; self.SH = SH
        self.lineS = self.axes.plot(lS,SH[iter0,:],'r',label="RichLucy estimate",linewidth=1)
        self.axes.legend()


    def update(self):
        value = self.slider.value()
        curr_iter = int(value)        
        if curr_iter > self.SH.shape[0]-1:
            curr_iter = self.SH.shape[0]-1
        self.iter_txt.set_text("iter %d" % curr_iter)
        self.lineS[0].set_data(self.lS,self.SH[curr_iter,:])
        self.canvas.draw()        

            
    def closeEvent(self,event):
        self.closed.emit()
        event.accept()
        



class main(QtGui.QWidget):
    wd = '' # working directory
    
    
    initial_maxiter = 500
    initial_MCruns = 100
    initial_checkUncCalc = 0
    initial_tau = 1e-5
    
   
    def __init__(self):#,parent=None):
        global version
#        QtGui.QWidget.__init__(self,parent)
        super(main,self).__init__()
        self.setupUi()
        self.resetAll()
       
        # connect the GUI objects to the respective methods        
        QtCore.QObject.connect(self.btn_load_bandpass, QtCore.SIGNAL("clicked()"), self.set_bandpass)
        QtCore.QObject.connect(self.btn_load_data, QtCore.SIGNAL("clicked()"), self.set_spectrum)
        QtCore.QObject.connect(self.startRL, QtCore.SIGNAL("clicked()"), self.callRichLucy)
        
        QtCore.QObject.connect(self.btn_cont,QtCore.SIGNAL("clicked()"), self.browseRL)
        QtCore.QObject.connect(self.btn_save2txt, QtCore.SIGNAL("clicked()"), self.save2file)
        QtCore.QObject.connect(self.btn_save2xls, QtCore.SIGNAL("clicked()"), self.save2Excel)
        QtCore.QObject.connect(self.btn_reset,QtCore.SIGNAL("clicked()"), self.resetAll)
#       
        QtCore.QObject.connect(self.btn_plot_curv,QtCore.SIGNAL("clicked()"),self.plotCurv)
        QtCore.QObject.connect(self.btn_plot_unc,QtCore.SIGNAL("clicked()"),self.plotUnc)    
        
#
        QtCore.QObject.connect(self.btn_plot_comp,QtCore.SIGNAL("clicked()"),self.plotComp)    
        QtCore.QObject.connect(self.btn_plot_ref,QtCore.SIGNAL("clicked()"),self.plotRef)    
        QtCore.QObject.connect(self.btn_load_ref,QtCore.SIGNAL("clicked()"),self.set_ref) 
        
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("clicked(QAbstractButton*)"), self.close)
        
        print "Richardson-Lucy bandpass correction graphical user interface version " + str(version) + "\n"
        print "Copyright S. Eichstaedt, F. Schmaehling (PTB) 2013"
        print "This software is licensed under a BSD-like license (see README)"
        print "This software comes without any warranty\n"        
        
        print "Load a file containing the bandpass function\n and a file containing a measured spectrum."
        print "Then hit >Start<!"        
        
    valueChanged = QtCore.pyqtSignal(int)
   
    def setupUi(self):
        self.resize(950, 800)
        self.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.setWindowTitle("Richardson-Lucy bandpass correction")
    # set font size and weight for all text        
        font = QtGui.QFont()        
        font.setBold(True)
        font.setWeight(75)
            
        self.main_layout = QtGui.QHBoxLayout()
#        self.lhs_layout = QtGui.QVBoxLayout()
        self.rhs_layout = QtGui.QVBoxLayout()        
        
    # left-hand side part of the GUI
        self.lhs = QtGui.QGroupBox()
        self.lhs.setFixedSize(QtCore.QSize(270,780))
        
# Generate group which contains the input data section    
        x = 10; y = 10; 
        width = 250;         
                    
        inputBox_height = 155
        parbox_height = 100
        setbox_height = 100
        button_height = 50
        inputBox2_height = 120
        plotBox_height = 60
        saveBox_height = 60
        buttonBox_height = 60
        margin = 10
        
        
        closeAction = QtGui.QAction('Close',self)
        closeAction.setShortcut('Crtl+Q')
        closeAction.setStatusTip('Close')
        closeAction.triggered.connect(self.close)
        

        self.inputBox = QtGui.QGroupBox(self.lhs)
        self.inputBox.setGeometry(QtCore.QRect(x,y,width,inputBox_height))
        y += inputBox_height + margin
        self.inputBox.setFont(font)
        self.inputBox.setObjectName("inputBox")
        self.inputBox.setTitle("Data files")
    # Generate text edits to show path to loaded data
        self.filename_bandpass = QtGui.QLineEdit(self.inputBox)
        self.filename_bandpass.setGeometry(QtCore.QRect(10, 40, 231, 20))
        self.filename_bandpass.setObjectName("filename_bandpass")
        self.filename_spectrum = QtGui.QLineEdit(self.inputBox)
        self.filename_spectrum.setGeometry(QtCore.QRect(10, 100, 231, 20))
        self.filename_spectrum.setObjectName("filename_spectrum")
    # Generate button to load bandpass data    
        self.btn_load_bandpass = QtGui.QPushButton(self.inputBox)
        self.btn_load_bandpass.setGeometry(QtCore.QRect(167, 65, 75, 23))
        self.btn_load_bandpass.setObjectName("btn_load_bandpass")
        self.btn_load_bandpass.setText("Browse...")
        self.btn_load_bandpass.setToolTip("Open file dialogue to load bandpass data from file")
    # Generate button to load spectrum data    
        self.btn_load_data = QtGui.QPushButton(self.inputBox)
        self.btn_load_data.setGeometry(QtCore.QRect(167, 125, 75, 23))
        self.btn_load_data.setObjectName("btn_load_data")
        self.btn_load_data.setText("Browse...")
        self.btn_load_data.setToolTip("Open file dialogue to load measured spectrum")
    # Generate explanatory text
        self.label_b = QtGui.QLabel(self.inputBox)
        self.label_b.setGeometry(QtCore.QRect(10, 20, 110, 16))
        self.label_b.setObjectName("label_b")        
        self.label_b.setText("bandpass function")
        self.label_s = QtGui.QLabel(self.inputBox)
        self.label_s.setGeometry(QtCore.QRect(10, 80, 110, 16))
        self.label_s.setObjectName("label_s")
        self.label_s.setText("spectrum")    
#        self.lhs_layout.addWidget(self.inputBox)
        
# Generate group box containing the parameters    
        self.parBox = QtGui.QGroupBox(self.lhs)
        self.parBox.setFont(font)
        self.parBox.setObjectName("parBox")
        self.parBox.setTitle("Parameters")        
        self.parBox.setGeometry(QtCore.QRect(x,y,250,parbox_height))
        y += parbox_height + margin
    # Generate labels with explanation of parameters
        self.label_max = QtGui.QLabel(self.parBox)
        self.label_max.setGeometry(QtCore.QRect(15, 20, 91, 16))
        self.label_max.setObjectName("label_max")
        self.label_max.setText("max. iterations")        
    # Generate text edit fields for the parameters    
        self.maxiter_Edit = QtGui.QLineEdit(self.parBox)
        self.maxiter_Edit.setGeometry(QtCore.QRect(170, 20, 71, 20))
        self.maxiter_Edit.setObjectName("maxiter_Edit")
        self.maxiter_Edit.setText(str(self.initial_maxiter))
        self.maxiter_Edit.setToolTip("Maximum number of iterations of the "\
                "Richardson-Lucy method.")
    # Generate checkbox for uncertainty evaluation
        self.label_unc = QtGui.QLabel(self.parBox)
        self.label_unc.setGeometry(QtCore.QRect(15,50,140,20))        
        self.label_unc.setText("evaluate uncertainties")
        self.uncertainty = QtGui.QCheckBox(self.parBox)
        self.uncertainty.setGeometry(QtCore.QRect(170,50,15,15))
        self.uncertainty.setChecked(self.initial_checkUncCalc)
        self.uncertainty.setToolTip("If active, then uncertainties will be "\
                "evaluated using the GUM-S2 Monte Carlo method.\n "\
                "Caution: This can take a long time and the GUI will freeze "\
                "for some seconds or minutes.")
    # Generate edit for number of Monte Carlo runs
        self.label_MCruns = QtGui.QLabel(self.parBox)
        self.label_MCruns.setGeometry(QtCore.QRect(35,73,120,20))
        self.label_MCruns.setText("Monte Carlo runs")
        self.MCruns = QtGui.QLineEdit(self.parBox)
        self.MCruns.setGeometry(QtCore.QRect(180,75,61,20))
        self.MCruns.setText(str(self.initial_MCruns))
        self.MCruns.setToolTip("The number of Monte Carlo trials for the\n "\
                "evaluation of uncertainties")

# Generate box for RL settings
        self.setBox = QtGui.QGroupBox(self.lhs)
        self.setBox.setFont(font)
        self.setBox.setObjectName("setBox")
        self.setBox.setTitle("Settings")
        self.setBox.setGeometry(QtCore.QRect(x,y,250,setbox_height))
        y += setbox_height + margin
        
# Generate dropdown for stopping rule        
        self.choose_stop = QtGui.QComboBox(self.setBox)
        self.choose_stop.setGeometry(QtCore.QRect(100,20,145,20))
        self.choose_stop.addItem("discrepancy principle")
        self.choose_stop.addItem("curv(progress)")
        self.choose_stop.addItem("curv(entropy)")        
        self.choose_stop.addItem("None")
        self.label_stoprule = QtGui.QLabel(self.setBox)
        self.label_stoprule.setGeometry(QtCore.QRect(15,20,120,20))
        self.label_stoprule.setText("stopping rule")

# Generate dropdown for smoothing method
        self.choose_smooth = QtGui.QComboBox(self.setBox)
        self.choose_smooth.setGeometry(QtCore.QRect(140,50,105,20))
        self.choose_smooth.addItem("None")
        self.choose_smooth.addItem("Tikhonov")
        self.choose_smooth.addItem("Total-Variation")
        self.label_smooth = QtGui.QLabel(self.setBox)
        self.label_smooth.setGeometry(QtCore.QRect(15,50,120,20))
        self.label_smooth.setText("smoothing method")
        self.tau_label = QtGui.QLabel(self.setBox)
        self.tau_label.setGeometry(QtCore.QRect(15,75,125,20))
        self.tau_label.setText("smoothing parameter")        
        self.smooth_tau = QtGui.QLineEdit(self.setBox)
        self.smooth_tau.setGeometry(QtCore.QRect(160,75,85,20))
        self.smooth_tau.setText(str(self.initial_tau))
        self.smooth_tau.setToolTip("Smoothing parameter value to weight penalization of non-smooth"\
                        "values. Too large values may cause instabilities in the RichLucy iterations!")

# Generate dropdown for stopping rule        
        self.choose_stop = QtGui.QComboBox(self.setBox)
        self.choose_stop.setGeometry(QtCore.QRect(100,20,145,20))
        self.choose_stop.addItem("discrepancy principle")
        self.choose_stop.addItem("curv(progress)")
        self.choose_stop.addItem("curv(entropy)")        
        self.choose_stop.addItem("None")
        self.label_stoprule = QtGui.QLabel(self.setBox)
        self.label_stoprule.setGeometry(QtCore.QRect(15,20,120,20))
        self.label_stoprule.setText("stopping rule")

 # Generate button to call RichLucy method        
        self.startRL = QtGui.QPushButton(self.lhs)
        self.startRL.setFont(font)
        self.startRL.setObjectName("startRL")
        self.startRL.setText("Start RichLucy")
        self.startRL.setToolTip("Start the Richardson-Lucy deconvolution")
        self.startRL.setGeometry(QtCore.QRect(20,y,160,button_height))        
        self.startRL.setFixedSize(QtCore.QSize(110,30))
        
        
# Generate button to continue calculations
        self.btn_cont = QtGui.QPushButton(self.lhs)
        self.btn_cont.setGeometry(QtCore.QRect(140,y,160,button_height))
        self.btn_cont.setFixedSize(QtCore.QSize(110,30))
        self.btn_cont.setFont(font)
        self.btn_cont.setObjectName("btn_cont")
        self.btn_cont.setText("Browse iterations")
        self.btn_cont.setEnabled(True)
        self.btn_cont.setToolTip("Open new plot window to browse the RL iterations.")
        y += button_height + margin

        
# Generate box to load reference spectrum        
        y += 10
        self.inputBox2 = QtGui.QGroupBox(self.lhs)
        self.inputBox2.setGeometry(QtCore.QRect(10,y,250,inputBox2_height))
        y += inputBox2_height + margin
        self.inputBox2.setFont(font)
        self.inputBox2.setObjectName("inputBox2")
        self.inputBox2.setTitle("Reference spectrum")
    # Generate text edits to show path to loaded ref spectrum
        self.filename_ref = QtGui.QLineEdit(self.inputBox2)
        self.filename_ref.setGeometry(QtCore.QRect(10, 25, 231, 20))
        self.filename_ref.setObjectName("filename_ref")        
    # Generate button to load ref spectrum data    
        self.btn_load_ref = QtGui.QPushButton(self.inputBox2)
        self.btn_load_ref.setGeometry(QtCore.QRect(167, 55, 75, 23))
        self.btn_load_ref.setObjectName("btn_load_ref")
        self.btn_load_ref.setText("Browse...")
        self.btn_load_ref.setToolTip("Open file dialogue to load reference spectrum")
    # Generate button to plot ref spectrum
        self.btn_plot_ref = QtGui.QPushButton(self.inputBox2)
        self.btn_plot_ref.setGeometry(QtCore.QRect(10, 90, 111, 23))
        self.btn_plot_ref.setText("plot ref. spectrum")
        self.btn_plot_ref.setDisabled(1)
        self.btn_plot_ref.setToolTip("Plot reference spectrum")
    # Generate button to compare ref to estimated spectrum
        self.btn_plot_comp = QtGui.QPushButton(self.inputBox2)
        self.btn_plot_comp.setGeometry(QtCore.QRect(125, 90, 111, 23))
        self.btn_plot_comp.setText("compare to est.")
        self.btn_plot_comp.setDisabled(1)
        self.btn_plot_comp.setToolTip("Plot reference spectrum and estimated spectrum")

        
# Generate group of plot buttons
        self.plotBox = QtGui.QGroupBox(self.lhs)
        self.plotBox.setGeometry(QtCore.QRect(10,y,250,plotBox_height))
        y += plotBox_height + margin
        self.plotBox.setFont(font)
        self.plotBox.setObjectName("plotBox")
        self.plotBox.setTitle("Additional plots")        
        
        self.btn_plot_curv = QtGui.QPushButton(self.plotBox)
        self.btn_plot_curv.setGeometry(QtCore.QRect(10, 20, 111, 23))
        self.btn_plot_curv.setText("stopping rule")
        self.btn_plot_curv.setDisabled(1)
        
        self.btn_plot_unc = QtGui.QPushButton(self.plotBox)
        self.btn_plot_unc.setGeometry(QtCore.QRect(125, 20, 111, 23))
        self.btn_plot_unc.setText("uncertainties")
        self.btn_plot_unc.setDisabled(1)

# Generate group of save buttons
        self.saveBox = QtGui.QGroupBox(self.lhs)
        self.saveBox.setGeometry(QtCore.QRect(10,y,250,saveBox_height))
        y += saveBox_height + margin
        self.saveBox.setFont(font)
        self.saveBox.setObjectName("saveBox")
        self.saveBox.setTitle("Save results")        
        
        self.btn_save2txt = QtGui.QPushButton(self.saveBox)
        self.btn_save2txt.setGeometry(QtCore.QRect(10, 20, 111, 23))
        self.btn_save2txt.setText("save to txt/csv")
        
        self.btn_save2xls = QtGui.QPushButton(self.saveBox)
        self.btn_save2xls.setGeometry(QtCore.QRect(125, 20, 111, 23))
        self.btn_save2xls.setText("save to Excel")
        
        
    # Generate close button
        self.buttonBox = QtGui.QDialogButtonBox(self.lhs)
        self.buttonBox.setGeometry(QtCore.QRect(180, y, 81, buttonBox_height))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Close)
        self.buttonBox.setObjectName("buttonBox")
    # Generate reset button
        self.btn_reset = QtGui.QPushButton(self.lhs)
        self.btn_reset.setGeometry(QtCore.QRect(100, y+18, 81, 23))
        self.btn_reset.setObjectName("btn_reset")
        self.btn_reset.setText("Reset")
        
        
# Generate widget to plot bandpass function
        self.bandpass_widget = QtGui.QWidget()
        self.bandpass_fig = Figure()
        self.bandpass_canvas = Canvas(self.bandpass_fig)
        self.bandpass_canvas.setParent(self.bandpass_widget)        
        self.mpl_bandpass = self.bandpass_fig.add_subplot(111)
        self.bandpass_fig.suptitle("bandpass function")
        self.bandpass_widget.setObjectName("mpl_bandpass")
        self.bandpass_toolbar = NavigationToolbar(self.bandpass_canvas, self.bandpass_widget)
        self.layout_b = QtGui.QVBoxLayout()
        self.layout_b.addWidget(self.bandpass_canvas)
        self.layout_b.addWidget(self.bandpass_toolbar)
        self.bandpass_widget.setLayout(self.layout_b)
        self.rhs_layout.addWidget(self.bandpass_widget)

       
# Generate widget to plot spectrum and estimation result
        self.spectrum_widget = QtGui.QWidget()
        self.spectrum_fig = Figure()
        self.spectrum_canvas = Canvas(self.spectrum_fig)
        self.spectrum_canvas.setParent(self.spectrum_widget)
        self.spectrum_fig.suptitle("spectrum")
        self.mpl_spectrum = self.spectrum_fig.add_subplot(111)
        self.spectrum_widget.setObjectName("mpl_spectrum")
        self.spectrum_toolbar = NavigationToolbar(self.spectrum_canvas, self.spectrum_widget)
        self.layout_s = QtGui.QVBoxLayout()
        self.layout_s.addWidget(self.spectrum_canvas)
        self.layout_s.addWidget(self.spectrum_toolbar)
        self.spectrum_widget.setLayout(self.layout_s)
        self.rhs_layout.addWidget(self.spectrum_widget)

# Generate general internal connections
        
        
        self.main_layout.addWidget(self.lhs)
        self.main_layout.addLayout(self.rhs_layout)
        self.setLayout(self.main_layout)        
        
       
    def reset(self):
        # reset all elements to their initial values and appearance
        self.filename_bandpass.setText('')
        self.filename_spectrum.setText('')
        self.filename_ref.setText('')
        self.maxiter_Edit.setText(str(self.initial_maxiter))
        self.MCruns.setText(str(self.initial_MCruns))
        self.uncertainty.setChecked(self.initial_checkUncCalc)
        self.mpl_bandpass.axes.cla()
        self.mpl_spectrum.axes.cla()
        self.btn_plot_curv.setDisabled(1)
        self.btn_plot_unc.setDisabled(1)
        self.btn_plot_ref.setDisabled(1)
        self.btn_plot_comp.setDisabled(1)


    def resetAll(self):
        self.bandpass = None
        self.bandpass_orig = None
        self.lambda_b = None
        self.lb_orig = None
        self.unc_bandpass = None
        self.shift = None
        self.interp_bandpass = False
        
        self.M = None
        self.lambda_M = None
        self.unc_M = None
        self.lM_orig = None
        self.M_orig = None  
        self.interp_M = False
        
        self.lambda_S = None
        self.Shat = None
        self.SH = None
        self.uShat = None
        self.UShat = None
        self.interp_Shat = None
                
        self.Shat0 = None
        
        self.RMS = None
        self.curv = None
        
        self.Ref = None
        self.lambda_Ref = None
        
        
        self.reset()


    def display(self,text):
        """
        print out text message if checkbox is checked
        """        
        if display_text_output:
            sys.stdout.write(text)

    def displayPars(self,text,pars):
        """
        print out text message containing parameters if checkbox is checked
        """
        if display_text_output:
            print text % (pars)


    def set_bandpass(self):        
        """
        
        Function to load bandpass data from user supplied text file.
        It checks whether the wavelength scale axis is monotonically increasing,
        whether it is equidistant and it normalizes the bandpass function to 
        integrate to 1. If the data is not sampled equidistantly, a new
        axis is defined and integration is carried out.
        
        """
        
        self.display("\n------- Loading bandpass data file -------\n")

        data,name = self.FileDialog()
            
        self.filename_bandpass.setText(name)
        
        self.lb_orig = data[:,0]
        self.bandpass_orig = data[:,1]

        self.lambda_b,self.bandpass,self.interp_bandpass = RL.fix_bandpass(self.lb_orig,self.bandpass_orig,display_text_output)        

        self.delta = self.lambda_b[1]-self.lambda_b[0]
        
        m,n = data.shape    
        if n > 2: # data contains uncertainties            
            self.unc_bandpass = np.abs(data[:,2:])
        else:
            self.display("Bandpass data file contains no uncertainties. Setting all to zero...\n")
            self.unc_bandpass = np.zeros(self.bandpass.shape)
            
        # plot in GUI widget
        self.mpl_bandpass.axes.cla()
        if self.interp_bandpass:
            nb = np.trapz(self.bandpass_orig,self.lb_orig)            
            self.mpl_bandpass.axes.plot(self.lambda_b,self.bandpass,'b',\
                    self.lb_orig,self.bandpass_orig/nb,'r.')
            if np.abs(1-nb) > 1e-4:
                self.mpl_bandpass.axes.legend(("interpolated","data (normalized)"),loc='best')
            else:
                self.mpl_bandpass.axes.legend(("interpolated","data"),loc='best')
        else:
            self.mpl_bandpass.axes.plot(self.lambda_b,self.bandpass,'b')           
        self.bandpass_canvas.draw()
        # disable plot of results (if any are available)
        self.btn_plot_curv.setDisabled(1)
        # allow plot of input uncertainties
        self.btn_plot_unc.setEnabled(1)
        
        self.display("----------------------------------------------\n")
        
        

    def set_spectrum(self):
        """
        Function to load measured spectrum data from file. This function is
        called when the "Browse" button for measured spectrum is clicked.
        """

        if not isinstance(self.lambda_b,np.ndarray):
            print "Error: Bandpass has to be loaded before measured spectrum."
            return
        
        self.display("\n------- Loading measured spectrum data file --------\n")
        data,name = self.FileDialog()            
        self.filename_spectrum.setText(name)
        
        self.lM_orig = data[:,0]
        self.M_orig = data[:,1]        
        self.lambda_M,self.M,interp_bM = RL.fix_inputs(self.lambda_b,self.bandpass,self.lM_orig,self.M_orig,display=display_text_output,returnDiagnostics=True)[2:]
        self.interp_M = interp_bM[1]       
        self.lambda_S = self.lM_orig[:]
        
        m,n = data.shape
        if n > 2:
            self.unc_M = np.abs(data[:,2:])
        else:
            self.display("No uncertainties found in measured spectrum data file. Setting all to zero.\n") 
            self.unc_M = np.zeros(self.M.shape)
            
        self.mpl_spectrum.axes.cla()
        if self.interp_M:
            self.mpl_spectrum.axes.plot(self.lambda_M,self.M,'b',self.lM_orig,self.M_orig,'r.')
            self.mpl_bandpass.axes.legend(("interpolated","data"),loc='best')
        else:            
            self.mpl_spectrum.axes.plot(self.lambda_M,self.M)
        self.spectrum_canvas.draw()
    # disable plot of results if any
        self.btn_plot_curv.setDisabled(1)
    # allow plot of input uncertainties
        self.btn_plot_unc.setEnabled(1)
        self.display("----------------------------------------------\n")


    def set_ref(self):
        """
        This functions is called when the "Browse" button for the reference
        spectrum is clicked. It loads a user-supplied file containing a reference
        spectrum to which the estimate can be compared to.
        """

        self.display("\n------- Loading reference spectrum data file --------\n")
        try:
            data,name = self.FileDialog()
        except ValueError:
            print "Reference spectrum data file could not be loaded."   
            return
            
        self.filename_ref.setText(name)        

        m,n = data.shape
 
        self.lambda_Ref = data[:,0]
        self.Ref = data[:,1]                
        # allow plot of ref spectrum
        self.btn_plot_ref.setEnabled(1)
        if isinstance(self.Shat,np.ndarray):
            self.btn_plot_comp.setEnabled(1)
        
        self.display("-------------------------------------------\n")


    def FileDialog(self):
        """
        Function to open dialog and load data from text file        
        """
        
        self.btn_cont.setEnabled(0)        
        if self.wd=='':
            curr_wd = str(QtCore.QDir.currentPath)
        else:
            curr_wd = self.wd
        name = QtGui.QFileDialog.getOpenFileName(self,filter="*.txt;*.csv",directory=curr_wd)        
        name = unicode(name)        
        if not isfile(name):
            print "File not found or error while trying to open it."
            raise ValueError

        self.wd = name
        data = RL.LoadFromFile(name)        
        return data, name



    def set_stoprule(self):
        Meas = self.M
        b = self.bandpass[::-1]*self.delta
        ind = np.nonzero(Meas>1e-3*Meas.max())[0][0]
        noise = Meas[:ind].std()
        if self.choose_stop.currentIndex()==0:            
            # using larger value than estimated noise because it typically gives better results
            self.stoprule = sr.discrepancy(Meas,b,noise*5)
        elif self.choose_stop.currentIndex()==1:
            self.stoprule = sr.curv_progress(Meas,b,noise)
        elif self.choose_stop.currentIndex()==2:
            self.stoprule = sr.curv_entropy(Meas,b,noise,self.delta)
        else:
            self.stoprule = None
            
    def set_smoothmethod(self):
        delta = self.delta
        regtau = float(str(self.smooth_tau.text()))
        if self.choose_smooth.currentIndex()==0:
            self.smooth_method = None
        elif self.choose_smooth.currentIndex()==1:
            self.smooth_method = sm.Tikhonov(delta,regtau)
        elif self.choose_smooth.currentIndex()==2:
            self.smooth_method = sm.TotalVar(delta,regtau)


    def browseRL(self):
        self.new = BrowseRL(self.lM_orig,self.M_orig,self.lambda_M,self.SH,parent=self)
        self.new.show()


    """
    The functions to generate plot figures all use the same method self.plot()    
    """
    
    def plotCurv(self):               
        if self.choose_stop.currentIndex()==3:
            return

        self.fig_curv = PlotWindow(self)
        self.fig_curv.show()
        maxiter = int(str(self.maxiter_Edit.text()))
        iters = np.arange(maxiter)

        if self.choose_stop.currentIndex()==0:
            self.fig_curv.changeTitle("Discrepancy principle")
            self.fig_curv.semilogx(iters,np.log10(self.stoprule.crit_measure))
            self.fig_curv.axvline(self.stoprule.indopt)
        if self.choose_stop.currentIndex()==1:
            self.fig_curv.changeTitle("curvature of estimation progress")
            self.fig_curv.semilogx(10**self.stoprule.q,self.stoprule.crit_measure)
            self.fig_curv.axvline(self.stoprule.indopt)
        if self.choose_stop.currentIndex()==2:
            self.fig_curv.changeTitle("curvature of estimation entropy")
            self.fig_curv.semilogx(iters,self.stoprule.crit_measure)
            self.fig_curv.axvline(self.stoprule.indopt)

        
    def plotUnc(self):
        self.fig_uncb = PlotWindow(self)
        self.plot(self.fig_uncb,"uncertainty of bandpass",self.lb_orig,self.unc_bandpass)        
        
        self.fig_uncM = PlotWindow(self)
        if len(self.unc_M.shape)>1:
            self.plot(self.fig_uncM,"uncertainty of measured spectrum",self.lM_orig,np.sqrt(np.diag(self.unc_M)))
        else:
            self.plot(self.fig_uncM,"uncertainty of measured spectrum",self.lM_orig,self.unc_M)
        
        if isinstance(self.uShat,np.ndarray):
            self.fig_uncSh= PlotWindow(self)
            self.plot(self.fig_uncSh,"uncertainty of estimated spectrum",self.lambda_S,self.uShat)


    def plotRef(self):
        self.fig_Ref = PlotWindow(self)
        self.plot(self.fig_Ref,"reference spectrum",self.lambda_Ref,self.Ref)

       
    def plotComp(self):
        self.fig_comp = PlotWindow(self)
        self.plot(self.fig_comp,"Estimated spectrum (blue) vs reference spectrum (red)",\
                  self.lambda_S,self.Shat,x2=self.lambda_Ref,y2=self.Ref,col1='b',col2='r')
       


    def plot(self,fighandle,title,x1,y1,x2=None,y2=None,col1='b',col2='r'):
        """
        Generate a plot of provided values in user-specified figure        
        """        
        if isinstance(fighandle,PlotWindow):        
            fighandle.show()
            fighandle.changeTitle(title)
            fighandle.plot(x1,y1,x2=x2,y2=y2,col1=col1,col2=col2)



    def save2file(self):
        
        if not isinstance(self.Shat,np.ndarray):
            raise ValueError("Nothing to save - call RichLucy first by clicking <Start>")
            
        if isinstance(self.UShat,np.ndarray):
            saveUnc = True
        else:
            saveUnc = False
            
        name = str(QtGui.QFileDialog.getSaveFileName(self,"Save results to file", "*.txt"))
        name = unicode(name)
        if name != '':
            if saveUnc:
                reply = QtGui.QMessageBox.question(self, "Save covariances?",
                                "Uncertainties associated with the estimate (if calculated) can be stored as covariance matrix or as simple standard point-wise uncertainties.\n Do you want to store covariances?",
                                QtGui.QMessageBox.StandardButtons(QtGui.QMessageBox.Yes).__or__(QtGui.QMessageBox.No),
                                QtGui.QMessageBox.No)        
            header1 = "# Result of Richardson-Lucy deconvolution calculated using the RichLucy GUI version " + repr(version) + "\n"
            if self.interp_bandpass or self.interp_M or self.interp_Shat:
                header2 = "# Estimation did require interpolation of data."
            else:
                header2 = ""
            header3 = "# Suggested shift of wavelength scale is " + repr(self.shift) + "\n"
            if saveUnc:                
                if reply == QtGui.QMessageBox.No:        
                    header4 = "# wavelengths | estimated spectrum | standard point-wise uncertainties\n"        
                else:
                    header4 = "# wavelengths | estimated spectrum | uncertainties (covariance matrix)\n"
            else:
                header4 = "# wavelengths | estimated spectrum\n"
                
            header = header1 + header2 + header3 + header4
            
            datei = open(name,'w')
            datei.write(header)
            if saveUnc:
                if reply == QtGui.QMessageBox.No:
                    Results = np.vstack((self.lambda_S,self.Shat,self.uShat))
                else:
                    Results = np.vstack((self.lambda_S,self.Shat,self.UShat))
            else:
                Results = np.vstack((self.lambda_S,self.Shat))
            Results = Results.transpose()
            np.savetxt(datei,Results,delimiter="\t")
            reply = QtGui.QMessageBox(self)
            reply.setText("Result saved successfully to " + name + ".")
            reply.exec_()
        else:
            print "Saving cancelled by user."


    def save2Excel(self):
        import openpyxl as xl
        from openpyxl.cell import get_column_letter as get
        from openpyxl.styles import Font,Style,NumberFormat
        from datetime import datetime

        if not isinstance(self.Shat,np.ndarray):
            raise ValueError("Nothing to save - call RichLucy first by clicking <Start>")
            
        if isinstance(self.UShat,np.ndarray):
            saveUnc = True
        else:
            saveUnc = False
            
        filename = str(QtGui.QFileDialog.getSaveFileName(self,"Save results to file", "*.xlsx"))
        filename = unicode(filename)
        header = []
        if filename != '':
            if saveUnc:
                reply = QtGui.QMessageBox.question(self, "Save covariances?",
                                "Uncertainties associated with the estimate (if calculated) can be stored as covariance matrix or as simple standard point-wise uncertainties.\n Do you want to store covariances?",
                                QtGui.QMessageBox.StandardButtons(QtGui.QMessageBox.Yes).__or__(QtGui.QMessageBox.No),
                                QtGui.QMessageBox.No)        
            header.append(u"Result of Richardson-Lucy deconvolution calculated using the RichLucy GUI version " + repr(version))
            if self.interp_bandpass or self.interp_M or self.interp_Shat:
                header.append(u"Estimation did require interpolation of data.")
            header.append(u"Suggested shift of wavelength scale is " + repr(self.shift))
            if saveUnc:                
                if reply == QtGui.QMessageBox.No:        
                    names = [u"wavelengths",u"estimated spectrum",u"standard point-wise uncertainties"]
                else:
                    names = [u"wavelengths",u"estimated spectrum",u"uncertainties (covariance matrix)"]
            else:
                names = [u"wavelengths",u"estimated spectrum"]
                        
        book = xl.Workbook()
        sh = book.active
        sh.title  = "RichLucy results"
        
        font = Font(bold=True)
        style = Style(font=font)
        dstyle= Style(font=font,number_format=NumberFormat('yyyy-mm-dd'))
        nstyle =Style(number_format=NumberFormat('0.000000E+00'))
        
        def write(row,col,val,cstyle): 
            sh.cell('%s%s'%(get(col),row)).value = val
            sh.cell('%s%s'%(get(col),row)).style = cstyle
            
        
        counter = 1
        if isinstance(header,list):
            for head in header:
                write(counter,1,head,style)
                counter+=1
        else:
            write(counter,1,header,style)
            counter+=1
    
        if saveUnc:
            if reply == QtGui.QMessageBox.No:
                Results = np.vstack((self.lambda_S,self.Shat,self.uShat))
            else:
                Results = np.vstack((self.lambda_S,self.Shat,self.UShat))
        else:
            Results = np.vstack((self.lambda_S,self.Shat))
        Results = Results.transpose()
    
        write(counter,1,"file generated",style)
        write(counter,2,datetime.now(),dstyle)
        counter+=2
    
        for ind,name in zip(range(Results.shape[1]),names):
            write(counter,ind+1,name,style)
        counter+=1
        
        for k in range(Results.shape[0]):        
            for l in range(Results.shape[1]):
                write(counter+k,l+1,Results[k,l],nstyle)

        for l in range(1,Results.shape[1]+1):
            sh.column_dimensions[get(l)].width = 20
            
        book.save(filename)
        print "Excel document with %d lines saved successfully to %s" % (counter+Results.shape[0],filename)



########################################  main routine -- call of RichLucy ################################


    def callRichLucy(self):
        
        if not isinstance(self.M,np.ndarray) or not isinstance(self.bandpass,np.ndarray):
            reply = QtGui.QMessageBox(self)
            reply.setText("Please load bandpass data and spectrum measurement data before calling Richardson-Lucy iteration.")
            reply.exec_()
            return
       
        self.Shat0 = None       
        self.btn_cont.setEnabled(0)
       
        self.mpl_spectrum.axes.cla()
        self.spectrum_canvas.draw()

        if self.uncertainty.isChecked(): # evaluate uncertainties by Monte Carlo
            if np.sum(self.unc_bandpass)==0 and np.sum(self.unc_M)==0: # all uncertainties equal to zero
                raise ValueError("The data loaded from the files did not contain any uncertainties.\n"\
                        "Evaluation of uncertainty is thus not available.")
            else:
                self.RichLucyUncertainty()
        else:
            self.RichLucyEstimation()
        

    def RichLucyEstimation(self):
        """
        This function calls the RichLucy deconvolution method from the RichLucy.py module
        when no uncertainty evaluation is requested by the user.
        """        
        
        self.display("\n-------- RichLucy deconvolution -----------\n")        
        
        self.set_stoprule()
        self.set_smoothmethod()

        

        self.Shat,self.SH = RL.RichLucy(self.bandpass,self.delta,self.M,\
                                maxiter=int(str(self.maxiter_Edit.text())),\
                                display=display_text_output,stoprule=self.stoprule,\
                                smooth_method=self.smooth_method,returnAll=True)

        if self.interp_M:
            spl = interp1d(self.lambda_M,self.Shat,bounds_error=False,kind="linear")
            Shat = spl(self.lM_orig)
            Shat[np.isnan(Shat)]=0.0; Shat[np.isinf(Shat)]=0.0
            self.Shat = Shat
            self.mpl_spectrum.axes.plot(self.lambda_M,self.M,'b-',label="interpolated meas.")
            self.mpl_spectrum.axes.plot(self.lM_orig,self.M_orig,'ro',label="original meas.")
            self.mpl_spectrum.axes.plot(self.lambda_S,self.Shat,'go',label="estimated spectrum")
            self.mpl_spectrum.axes.legend(loc="best")
            self.spectrum_canvas.draw()
        else:
            self.mpl_spectrum.axes.plot(self.lambda_M,self.M,'b-',label="measured spectrum")
            self.mpl_spectrum.axes.plot(self.lambda_S,self.Shat,'go',label="estimated spectrum")
            self.mpl_spectrum.axes.legend(loc="best")
            self.spectrum_canvas.draw()
                    
        
        self.btn_cont.setEnabled(True)
        # enable plot of results
        self.btn_plot_curv.setEnabled(1)
        if self.btn_plot_ref.isEnabled():
            self.btn_plot_comp.setEnabled(1)
        
        self.display("------------- finished --------------\n")
        
        
        
    def RichLucyUncertainty(self):
        """
        This function carries out uncertainty evaluation for the Richardson-Lucy
        deconvolution by using the GUM Supplement 2 Monte Carlo method.
        
        """
        
        self.display("Starting Monte Carlo uncertainty evaluation for RichLucy deconvolution\n")
        print "Please note that during the Monte Carlo calculations the GUI may fail to respond."
        
        maxiter = int(str(self.maxiter_Edit.text()))
        display = display_text_output
        maxcpu  = maxcpu_for_uncertainty_calc
        runs = int(str(self.MCruns.text()))
        self.set_stoprule()
        
        self.lambda_S = self.lM_orig

        self.Shat,self.UShat = RL.RichLucyUncertainty(\
                        self.bandpass_orig,self.unc_bandpass,self.lb_orig,\
                        self.M_orig,self.unc_M,self.lM_orig,stoprule=self.stoprule,\
                        runs=runs,maxiter=maxiter,display=display,maxncpu=maxcpu)
        self.uShat = np.sqrt(np.diag(self.UShat))

        numNaN = np.count_nonzero(np.isnan(self.Shat))
        numNeg = np.count_nonzero(self.Shat < 0.0)
        if numNaN + numNeg > 0:
            print "RichLucy estimate contains %d NaN(s) and %d negative value(s). Setting all to zero." % (numNaN,numNeg)
        self.Shat[self.Shat<0] = 0.0
        self.Shat[np.nonzero(np.isnan(self.Shat))] = 0.0

        self.mpl_spectrum.axes.plot(self.lambda_M,self.M,\
                                       self.lambda_S,self.Shat,\
                                       self.lambda_S,self.uShat)
        self.mpl_spectrum.axes.legend(("meas.","est.","unc"))
        self.spectrum_canvas.draw()
        self.btn_cont.setEnabled(False)
        # enable plot of results
        if self.btn_plot_ref.isEnabled():
            self.btn_plot_comp.setEnabled(1)
        self.btn_plot_unc.setEnabled(1)
        
        self.display("------------- finished --------------\n")



        
if __name__=="__main__":
    app = QtGui.QApplication(sys.argv)
    myapp = main()
    myapp.show()
    sys.exit(app.exec_())

