# -*- coding: utf-8 -*-
"""
Created on Mon Aug 04 11:23:31 2014

Smoothing of the RichLucy estimates can be carried out by damping the update in each iteration.

The function RichLucy.RichLucy therefore uses

    Shat_new = smoothing.smooth(Shat)*(RL_update)

where RL_update is the standard, non-smoothed RL update and smoothing is a RichLucy.smoothing object.

Implemented methods

* Tikhonov
* TotalVar

Each smoothing method has to be derived from the smoothing class in RichLucy.py

@author: Sascha Eichstaedt
"""

import numpy as np
import RichLucy as RL


class Tikhonov(RL.smoothing):
    """
    Smoothing using Tikhonov regularization
    Therefore, each estimated spectrum in SH is weighted with the regularizer:
    :math:`1+\lambda \\nabla^2 (SH(k-1,:))`
    
    i.e., the smoothness of the previous iteration.
    
    Criterion introduced for RichLucy in
    G van Kempen, Image Restoration in Flourescence Microscopy, PhD Thesis,
    Technische Universiteit Delft, Holland, 1999   
    """
    
    name = "Tikhonov penalty"

    def __init__(self,delta,regtau=1e-5):
        self.delta = delta 
        self.tau = regtau
        self.output("Tikhonov smoothing with tau = %g\n" % regtau)
        
        
    def smooth(self,Shat):
        DSH = np.gradient(np.gradient(Shat,self.delta),self.delta)
        regShat = Shat/(1-2*self.tau*DSH)    
        regShat[regShat<0]=0.0    
        return regShat
    
    
class TotalVar(RL.smoothing):
    """
    smoothing based on total variation regularization
    Therefore, each estimated spectrum in SH is weighted with the regularizer:
    :math:`1-\lambda* div(\\nabla(SH(k-1,:)/\\vert\\nabla(SH(k-1,:))\\vert`
    Criterion introduced for RichLucy in
    N Dey, L Blanc-Feraud, C Zimmer, P Roux, Z Kam, J C Olivo-Marin and J Zerubia
    3D Microscopy Deconvolution using Richardson-Lucy Algorithm with Total Variation Regularization
    Rapport de Recherce, No 8272, July 2004, INRIA
   """
   
    name = "Total Variation"
    
    def __init__(self,delta,regtau=1e-5):
        self.delta = delta
        self.tau = regtau
        self.output("Total-Variation smoothing with tau=%g\n" % regtau)
        
    def smooth(self,Shat):
        DHS = np.gradient(Shat,self.delta)
        inds = np.nonzero(np.abs(DHS)>0)[0]
        regShat = np.zeros_like(Shat)
        regShat[inds] = Shat[inds]/(1.0 - self.tau*np.gradient( DHS[inds]/np.abs(DHS[inds]), self.delta) )
        regShat[regShat<0]=0.0
        
        return regShat

