# -*- coding: utf-8 -*-
"""
Created on Fri May 24 13:48:02 2013

Implementation of test cases for some of the modules in the RichLucy module

"""

import unittest
import RichLucy
import numpy

class TestModules(unittest.TestCase):
    
    def test1_simbandpass_smalldelta(self):
        print '\nTesting RichLucy.sim_bandpass for small step sizes'
        bandwidth = 20.0
        delta = 0.1
        skew = 1.0
        b,lamb = RichLucy.sim_bandpass(bandwidth,delta,skew)
        # lambda axis symmetric around zero
        self.assertAlmostEqual(numpy.max(numpy.diff(lamb)),\
                               numpy.min(numpy.diff(lamb)),\
                               places=5)
        # lambda axis is equidistant                               
        self.assertAlmostEqual(numpy.unique(numpy.diff(lamb))[0],\
                               delta,places=5)
        # lambda axis fits to bandwidth
        self.assertEqual(lamb[-1]-lamb[0],2*bandwidth)
        # returns normalized bandpass
        self.assertAlmostEqual(numpy.trapz(b,dx=delta),1.0,places=4)
        

    def test2_simbandpass_largedelta(self):
        print '\nTesting RichLucy.sim_bandpass for large step sizes'
        bandwidth = 20.0
        delta = 2
        skew = 1
        b,lamb = RichLucy.sim_bandpass(bandwidth,delta,skew)
        # lambda axis symmetric around zero        
        self.assertAlmostEqual(numpy.max(numpy.diff(lamb)),\
                               numpy.min(numpy.diff(lamb)),\
                               places=5)
        # lambda axis is equidistant                               
        self.assertEqual(numpy.unique(numpy.diff(lamb)),delta)
        # lambda axis fits to bandwidth
        self.assertEqual(lamb[-1]-lamb[0],2*bandwidth)
        # returns normalized bandpass
        self.assertAlmostEqual(numpy.trapz(b,dx=delta),1.0,places=4)
        
        
    def test3_simmeasurement(self):
        print '\nTesting RichLucy.sim_measurement'
        mu = 320
        sigma = 15
        Sfun = lambda x: numpy.exp(-(x-mu)**2/(2*sigma**2))
        bandwidth = 20
        delta = 0.1
        b,lamb = RichLucy.sim_bandpass(bandwidth,delta)
        M = RichLucy.sim_measurement(Sfun,lamb,b,bandwidth,delta)
        # simulated spectrum is non-negative
        self.assertGreaterEqual(numpy.min(M),0.0)
        # should return error msg if Sfun is not a function handle
        self.assertRaises(ValueError,RichLucy.sim_measurement,numpy.zeros((100,)),lamb,b,bandwidth,delta)
        

    def test4_RichLucy(self):
        print '\nTesting RichLucy.RichLucy without autostop'
        mu = 320
        sigma = 15
        Sfun = lambda x: numpy.exp(-(x-mu)**2/(2*sigma**2))
        bandwidth = 20
        delta = 0.1
        b,lamb = RichLucy.sim_bandpass(bandwidth,delta)
        M = RichLucy.sim_measurement(Sfun,lamb,b,bandwidth,delta)
        
        runs = 200
        Shat, SH = RichLucy.RichLucy(b,delta,M,maxiter=runs,autostop=False,display=False,calcCrit=False,returnAll=True)
        # estimated spectrum of same dimension as measured
        self.assertEqual(len(Shat),len(M))
        m,n = numpy.shape(SH)
        # dimension of SH is [runs,length(M)]
        self.assertEqual(m,runs+1)        
        self.assertEqual(n,len(M))
        # estimated spectrum is non-negative
        self.assertGreaterEqual(numpy.min(Shat),0.0)

        
    def test5_calcOptCurv(self):
        print '\nTesting RichLucy.calcOptCurv with simulated measurement'
        mu = 320
        sigma = 15
        Sfun = lambda x: numpy.exp(-(x-mu)**2/(2*sigma**2))
        bandwidth = 20
        delta = 0.1
        b,lamb = RichLucy.sim_bandpass(bandwidth,delta)
        M = RichLucy.sim_measurement(Sfun,lamb,b,bandwidth,delta)
        
        runs = 200
        Shat, SH = RichLucy.RichLucy(b,delta,M,maxiter=runs,autostop=False,display=False,calcCrit=False,returnAll=True)

        bdtilde = b[::-1]*delta
        results = RichLucy.calcOptCurv(SH,M,bdtilde,display=False,autostop=True)
        # should have three outputs
        self.assertEqual(len(results),3)
        # length of RMS vector values
        self.assertEqual(len(results[0]),runs)
        # length of curve vector values
        self.assertEqual(len(results[1]),len(results[0]))
        # last entry is integer value        
        self.assertIsInstance(results[2],(int,long))
        # resulting indopt is reasonable
        self.assertTrue(results[2]>0)
        self.assertTrue(results[2]<=runs)
        # should also run without optimal stopping
        results = RichLucy.calcOptCurv(SH,M,bdtilde,display=False,autostop=False)
        self.assertEqual(len(results),3)
        # resulting indopt should be the last iteration
        self.assertEqual(results[2],runs)
        # should return negative indopt if anything went wrong
        results = RichLucy.calcOptCurv(numpy.zeros_like(SH),M,bdtilde,display=False,autostop=True)
        self.assertEqual(len(results),3)
        self.assertTrue(results[2]<0)
        
    def test6_EstMeas(self):
        print '\nTesting RichLucy.EstMeas with simulated measurement'
        mu = 500
        sigma = 15
        Sfun = lambda x: numpy.exp(-(x-mu)**2/(2*sigma**2))
        bandwidth = 20
        delta = 0.1
        b,lamb = RichLucy.sim_bandpass(bandwidth,delta)
        lambda_min = 400    # min wavelength in spectrum measurement
        lambda_max = 600    # max wavelength in spectrum measurement
        lambda_Mess = numpy.arange(lambda_min+bandwidth,lambda_max-bandwidth,delta)
        M = RichLucy.sim_measurement(Sfun,lambda_Mess,b,bandwidth,delta)
    
        runs = 500
        Shat = RichLucy.RichLucy(b,delta,M,maxiter=runs,autostop=True,display=False,calcCrit=True)[0]
        
        Mhat,lambda_Mhat = RichLucy.EstMeas(Shat,b,lambda_Mess,lamb,display=False)
        self.assertEqual(len(Mhat),len(lambda_Mhat))
        self.assertEqual(len(Mhat),len(Shat))
        self.assertAlmostEqual(numpy.sum( (Mhat-M)**2 ), 0.0, 4)
        
    def test7_RLDeconv(self):
        print "\nTesting RichLucy.RichLucyDeconv with measured data"
        bfile = "Bandpass_Spaltbreite_2mm_620nm-645nm.csv"
        Mfile = "Kamera_2mm_Spaltbreite.csv"
        
        bdata = RichLucy.LoadFromFile(bfile)
        lb,b,ub = bdata[:,0],bdata[:,1],bdata[:,2]
        
        Mdata = RichLucy.LoadFromFile(Mfile)
        lM,M,uM = Mdata[:,0],Mdata[:,1],Mdata[:,2]
        
        Shat = RichLucy.RichLucyDeconv(lb,b,lM,M,display=False)
        Mhat = RichLucy.EstMeas(Shat,b,lM,lb,display=False)[0]
        
        self.assertEqual(len(M),len(Shat))
        self.assertAlmostEqual(numpy.sum( (Mhat-M)**2), 0.0, 1)
        
        
def Test_CalcOptFun(CalcOptFun):
    """
    
    Test any given method CalcOptFun which calculates an optimal stopping 
    from a series of RichLucy estimates for its consistency with the 
    assumptions made in the RichLucy method.
    
    These are:
        - 3 variables are provided as output: two vectors and the optimal iteration
        - the two vectors are of equal length (e.g. rms values and its curvature)
        - the optimal iteration is smaller or equal the actually performed one
        - if the method is called without autostop the resulting opt iter should be max
        - if all RichLucy estimates are zero (hence, something went wrong) return indopt<0
        
    Implicitly, the interface of the function CalcOptFun is checked:
        arguments are
            SH ....... the matrix of RichLucy intermediate results
            M ........ the vector of measured spectrum values
            b ........ the vector of line spread function values
        keyword arguments are
            display ... true/false - print out messages
            autostop .. true/false - calculate optimal stopping
            
    """
    
    # Is it a function?
    assert(hasattr(CalcOptFun,'__call__'))
        
    print 'Testing ' + CalcOptFun.__name__ + ' with simulated measurement'
    mu = 320
    sigma = 15
    Sfun = lambda x: numpy.exp(-(x-mu)**2/(2*sigma**2))
    bandwidth = 20
    delta = 0.1
    b,lamb = RichLucy.sim_bandpass(bandwidth,delta)
    M = RichLucy.sim_measurement(Sfun,lamb,b,bandwidth,delta)
    
    runs = 200
    Shat, SH = RichLucy.RichLucy(b,delta,M,maxiter=runs,autostop=False,display=False,calcCrit=False,returnAll=True)

    btilde = b[::-1]*delta
    results = CalcOptFun(SH,M,btilde,display=False,autostop=True)
    
    # should have three outputs
    assert(len(results)==3)
    # length of RMS vector values
    assert(len(results[0])==runs)
    # length of curve vector values
    assert(len(results[1])==len(results[0]))
    # last entry is integer value        
    assert(isinstance(results[2],(int,long)))
    # resulting indopt is reasonable
    assert(results[2]<=runs)
    # should also run without optimal stopping
    result = CalcOptFun(SH,M,btilde,display=False,autostop=False)
    assert(len(result)==3)
    # resulting indopt should be the last iteration
    assert(result[2]==runs)
    # should return negative indopt if anything went wrong
    result = CalcOptFun(numpy.zeros_like(SH),M,btilde,display=False,autostop=True)
    assert(len(result)==3)
    assert(result[2]<0)    
    
    print 'Done'


        
               
if __name__=="__main__":
        unittest.main()